﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm;

namespace MyMediaProject.RecommenderSystem.Algorithms.Utils
{
    /// <summary>
    /// Similarity model based on Keyword occurences
    /// </summary>
    public class KeywordSimilarityModel : BaseItemSimilarityModel, IItemSimilarityModel
    {
        /// <summary>
        /// Should all keywords per item be cached yes/no
        /// </summary>
        public bool CacheKeywords { get; set; }
        private Dictionary<int, HashSet<int>> itemKeywords = new Dictionary<int, HashSet<int>>();

        /// <summary>
        /// Constructor
        /// </summary>
        public KeywordSimilarityModel()
        {
            CacheKeywords = false;
        }

        /// <inheritdoc />
        protected override void computeSimilarities()
        {
            bool ckSetting = CacheKeywords;
            CacheKeywords = true;
            collectItemKeywords();
            base.computeSimilarities();
            CacheKeywords = ckSetting;
            if (!CacheKeywords) itemKeywords.Clear();
        }

        private HashSet<int> getItemKeywords(int itemID)
        {
            if (itemKeywords.ContainsKey(itemID)) return itemKeywords[itemID];
            IEntityRelationDataReader keywords = EntityRelationDataProvider.GetRelation(RelationType.ItemKeyword);
            keywords.Open();
            HashSet<int> kws = new HashSet<int>();
            while (keywords.Read())
            {
                if (keywords.GetInt32(0) == itemID) kws.Add(keywords.GetInt32(1));
            }
            if (CacheKeywords) itemKeywords[itemID] = kws;
            return kws;
        }

        private void collectItemKeywords()
        {
            IEntityRelationDataReader items = EntityRelationDataProvider.GetEntity(EntityType.CatalogItem);
            items.Open();
            while (items.Read())
            {
                itemKeywords[items.GetInt32(0)] = new HashSet<int>();
            }
            IEntityRelationDataReader keywords = EntityRelationDataProvider.GetRelation(RelationType.ItemKeyword);
            keywords.Open();
            while (keywords.Read())
            {
                itemKeywords[keywords.GetInt32(0)].Add(keywords.GetInt32(1));
            }
        }

        /// <inheritdoc />
        protected override void computeSimilarities(int itemId, int startIndex)
        {
            if (!similarityMap.ContainsKey(itemId)) similarityMap[itemId] = new Dictionary<int, ItemSimilarity>();

            IEntityRelationDataReader otherItems = EntityRelationDataProvider.GetEntity(EntityType.CatalogItem);
            otherItems.Open();
            for (int i = 0; i < startIndex; i++)
            {
                otherItems.Read();
            }

            HashSet<int> itemKWS = getItemKeywords(itemId);
            while (otherItems.Read())
            {
                int otherItemID = otherItems.GetInt32(0);
                if (otherItemID != itemId)
                {
                    HashSet<int> otherItemKWS = getItemKeywords(otherItemID);
                    int totalCount = itemKWS.Count + otherItemKWS.Count;
                    int matchCount = 0;
                    foreach (int keyword in itemKWS)
                    {
                        if (otherItemKWS.Contains(keyword)) matchCount++;
                    }
                    if (matchCount > 0)
                    {
                        // if matchCount > 0 then totalCount > 0, so no div 0
                        addItemSimilarity(itemId, otherItemID, matchCount / totalCount);
                    }
                }
            }
        }
    }
}
