﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Algorithms.Utils
{
    /// <summary>
    /// Helper class for storing dense matrices.
    /// </summary>
    /// <typeparam name="DATATYPE"></typeparam>
    /// <author>Steffen Rendle, University of Hildesheim</author>
    public class Matrix<DATATYPE>
    {
        /// <summary>
        /// Data array.
        /// </summary>
        public DATATYPE[] data;
        /// <summary>
        /// Dimension 1.
        /// </summary>
        public int dim1;
        /// <summary>
        /// Dimension 2.
        /// </summary>
        public int dim2;

        /// <summary>
        /// Initializes a new instance of the <see cref="Matrix&lt;DATATYPE&gt;"/> class.
        /// </summary>
        /// <param name="dim1">The dim1.</param>
        /// <param name="dim2">The dim2.</param>
        public Matrix(int dim1, int dim2)
        {
            this.dim1 = dim1;
            this.dim2 = dim2;
            data = new DATATYPE[dim1 * dim2];
        }
        /// <summary>
        /// Gets the specified x.
        /// </summary>
        /// <param name="x">The x.</param>
        /// <param name="y">The y.</param>
        /// <returns></returns>
        public DATATYPE get(int x, int y)
        {
            return data[x + y * dim1];
        }

        /// <summary>
        /// Sets the specified x.
        /// </summary>
        /// <param name="x">The x.</param>
        /// <param name="y">The y.</param>
        /// <param name="d">The d.</param>
        public void set(int x, int y, DATATYPE d)
        {
            data[x + y * dim1] = d;
        }
        /// <summary>
        /// Inits the specified d.
        /// </summary>
        /// <param name="d">The d.</param>
        public void init(DATATYPE d)
        {
            for (int i = 0; i < dim1 * dim2; i++)
            {
                data[i] = d;
            }
        }
        /// <summary>
        /// Adds the row.
        /// </summary>
        /// <param name="num_rows">The num_rows.</param>
        public void addRow(int num_rows)
        {
            if (num_rows > dim1)
            {
                DATATYPE[] data_new = new DATATYPE[num_rows * dim2 ];
                data.CopyTo(data_new, 0);
                data = data_new;
                dim1 = num_rows;
            }
        }
    }

    /// <summary>
    /// Utilities to work with matrices.
    /// </summary>
    public class MatrixUtils 
    {

        /// <summary>
        /// Inits the normal.
        /// </summary>
        /// <param name="matrix">The matrix.</param>
        /// <param name="mean">The mean.</param>
        /// <param name="stdev">The stdev.</param>
        /// <param name="dim1">The dim1.</param>
        static public void initNormal(Matrix<double> matrix, double mean, double stdev, int dim1)
        {
            Random rnd = Random.getInstance();
            for (int d2 = 0; d2 < matrix.dim2; d2++)
            {
                matrix.set(dim1, d2, rnd.NextNormal(mean, stdev));
            }
        }

        /// <summary>
        /// Inits the normal.
        /// </summary>
        /// <param name="matrix">The matrix.</param>
        /// <param name="mean">The mean.</param>
        /// <param name="stdev">The stdev.</param>
        static public void initNormal(Matrix<double> matrix, double mean, double stdev)
        {
            Random rnd = Random.getInstance();
            for (int x = 0; x < matrix.dim1; x++)
            {
                for (int y = 0; y < matrix.dim2; y++)
                {
                    matrix.data[x + y * matrix.dim1] = rnd.NextNormal(mean, stdev);
                }
            }
        }
        /// <summary>
        /// Incs the specified matrix.
        /// </summary>
        /// <param name="matrix">The matrix.</param>
        /// <param name="x">The x.</param>
        /// <param name="y">The y.</param>
        /// <param name="d">The d.</param>
        static public void inc(Matrix<double> matrix, int x, int y, double d)
        {
            matrix.data[x + y * matrix.dim1] += d;
        }
    }
}
