﻿using System;
using System.Linq;
using System.Configuration;

namespace MyMediaProject.RecommenderSystem.Core.Config
{
    internal static class ApplicationConfigurationUtility
    {
        private const String CONFIG_MANAGER_BINARY_LIBRARY_FILE_NAME = "MyMediaProject.Recommender.Core.dll";

        private static System.Configuration.Configuration appConfigObject;

        private static System.Configuration.Configuration GetApplicationConfigurationObject()
        {
            if (appConfigObject != null)
            {
                return appConfigObject;
            }

            try
            {
                appConfigObject = ConfigurationManager.OpenExeConfiguration(CONFIG_MANAGER_BINARY_LIBRARY_FILE_NAME);
            }
            catch (ArgumentException)
            {
                return null;
            }
            catch (ConfigurationErrorsException)
            {
                return null;
            }

            return appConfigObject;
        }

        /// <summary>
        /// Stores local application setting to the application specific storage
        /// </summary>
        /// <param name="settingName"></param>
        /// <param name="settingValue"></param>
        internal static void SaveApplicationSetting(string settingName, string settingValue)
        {
            // Get the configuration file.
            System.Configuration.Configuration appConfig = GetApplicationConfigurationObject();

            // if application configuration object can't be retrieved(e.g. in a unit testing context), this utility can't to anything
            if (appConfig == null)
            {
                return;
            }

            // if the setting exists already, remove it first (otherwise its value would be appended)
            if (appConfig.AppSettings.Settings.AllKeys.Contains(settingName))
            {
                RemoveApplicationSetting(settingName);
            }

            appConfig.AppSettings.Settings.Add(settingName, settingValue);
            _save_and_reload_application_config();
        }

        internal static void RemoveApplicationSetting(string settingName)
        {
            System.Configuration.Configuration appConfig = GetApplicationConfigurationObject();
            // if application configuration object can't be retrieved(e.g. in a unit testing context), this utility can't to anything
            if (appConfig == null)
            {
                return;
            }

            if (!appConfig.AppSettings.Settings.AllKeys.Contains(settingName))
            {
                return;
            }

            appConfig.AppSettings.Settings.Remove(settingName);
            _save_and_reload_application_config();
        }

        private static void _save_and_reload_application_config()
        {
            System.Configuration.Configuration appConfig = GetApplicationConfigurationObject();
            // if application configuration object can't be retrieved(e.g. in a unit testing context), this utility can't to anything
            if (appConfig == null)
            {
                return;
            }

            // Save the configuration file.
            appConfig.Save(ConfigurationSaveMode.Minimal);

            // Force a reload of the changed section.
            ConfigurationManager.RefreshSection("appSettings");
        }

        internal static String GetApplicationSetting(string settingName)
        {
            System.Configuration.Configuration appConfig = GetApplicationConfigurationObject();
            // if application configuration object can't be retrieved(e.g. in a unit testing context), this utility can't to anything
            if (appConfig == null)
            {
                return null;
            }

            if (!appConfig.AppSettings.Settings.AllKeys.Contains(settingName))
            {
                return null;
            }

            return appConfig.AppSettings.Settings[settingName].Value;
        }

    }
}
