﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EMIC.Design;

namespace MyMediaProject.RecommenderSystem.Core.Config
{
    /// <summary>
    /// This utility file holds 'shortcuts' to well known properties (in order to escape long access strings).
    /// </summary>
    public static partial class ConfigManager
    {
        /// <summary>
        /// Common DBConnectionString
        /// </summary>
        public static String DBConnectionString
        {
            get
            {
                ConfigManager.Initialize();
                return DefaultModuleProperties[ConfigConstants.PropertyNameDBConnectionString].Value;
            }
            set
            {
                ConfigManager.Initialize();
                DefaultModuleProperties[ConfigConstants.PropertyNameDBConnectionString].Value = value;
                TraceLog.Write(LogEventType.Information, "The value of the static DBConnectionString has been set to: '{0}.'", new object[] { value });
            }
        }

        /// <summary>
        /// Common DBConnectionString
        /// </summary>
        internal static String DefaultDBConnectionStringFormat
        {
            get
            {
                ConfigManager.Initialize();
                string value;
                try
                {
                    value = DefaultModuleProperties[ConfigConstants.PropertyNameDefaultDBConnectionStringFormat].Value;
                }
                catch (Exception e)
                {
                    throw new RecommenderConfigurationException(ConfigConstants.PropertyNameDefaultDBConnectionStringFormat, e);
                }
                Contract.Postcondition(!string.IsNullOrEmpty(value), "Configuration property, {0}, not set; verify configruation files are set up properly.", ConfigConstants.PropertyNameDefaultDBConnectionStringFormat);
                return value;
            }
            set
            {
                Contract.Precondition(!string.IsNullOrEmpty(value), "Configuration property, {0}, may not be set to empty or null.", ConfigConstants.PropertyNameDefaultDBConnectionStringFormat);
                ConfigManager.Initialize();
                DefaultModuleProperties[ConfigConstants.PropertyNameDefaultDBConnectionStringFormat].Value = value;
                TraceLog.Write(LogEventType.Information, "The value of the static DefaultDBConnectionStringFormat has been set to: '{0}.'", new object[] { value });
            }
        }

        /// <summary>
        /// DB Connection string that _might_ be used for executing tests
        /// </summary>
        public static String TestDBConnectionString
        {
            get
            {
                ConfigManager.Initialize();
                return DefaultModuleProperties[ConfigConstants.PropertyNameTestDBConnectionString].Value;
            }
        }

        /// <summary>
        /// Proxy string (might be empty)
        /// </summary>
        public static String ProxyString
        {
            get
            {
                ConfigManager.Initialize();
                return DefaultModuleProperties[ConfigConstants.PropertyNameProxyString].Value;
            }
            set
            {
                ConfigManager.Initialize();
                DefaultModuleProperties[ConfigConstants.PropertyNameProxyString].Value = value;
                TraceLog.Write(LogEventType.Information, "The value of the static ProxyString has been set to: '{0}.'", new object[] { value });
            }
        }
    }
}
