﻿using System;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Classes that want to act as a context source for the recommender context system, must derive
    /// from this class.
    /// </summary>
    public abstract class ContextSource
    {
        private RecommenderId id;
        private IContextBroker contextBroker;

        /// <summary>
        /// This property holds the id of the context broker class.
        /// </summary>
        /// <value>The id.</value>
        public RecommenderId Id
        {
            get { return id; }
        }

        /// <summary>
        /// This method is used internally by the framework to assign the context broker
        /// class reference. This reference can then be used to deliver context status updates.
        /// </summary>
        /// <param name="broker">The context broker object.</param>
        internal void SetContextBroker(IContextBroker broker)
        {
            this.contextBroker = broker;
        }

        /// <summary>
        /// This method can be used to deliver context status updates.
        /// </summary>
        /// <param name="contextStatus">The context status object, that reflects the new value of the context.</param>
        protected void UpdateContextStatus(ContextStatus contextStatus)
        {
            if (contextBroker != null)
            {
                contextBroker.ContextStatusChanged(contextStatus);
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ContextSource"/> class.
        /// </summary>
        /// <param name="id">The id of the context source.</param>
        protected ContextSource(RecommenderId id)
        {
            this.id = id;
        }
    }
}
