﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// A base class for all actions of <see cref="User"/> on <see cref="CatalogItem"/>.
    /// </summary>    
    public partial class UserAction
    {
         /// <summary>
        /// Initializes a new instance of the <see cref="UserAction"/> class.
        /// </summary>
        /// <param name="user">The user.</param>
        /// <param name="item">The item.</param>
        internal UserAction(User user, CatalogItem item)
            : this()
        {
            this.UserId = user.UserId;
            this.ItemId = item.ItemId;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UserAction"/> class.
        /// </summary>
        /// <param name="user">The user.</param>
        /// <param name="item">The item.</param>
        /// <param name="timestamp">The timestamp.</param>
        internal UserAction(User user, CatalogItem item, DateTime timestamp)
            : this(user, item)
        {
            this.Timestamp = timestamp;
        }

        /// <summary>
        /// Prepares the item for insert by reconciling any external tables.
        /// </summary>
        /// <param name="dbContext">The db context.</param>
        internal virtual void PrepareForInsert(MyMediaDataContext dbContext)
        {
        }

        /// <summary>
        /// A query which restricts the actions to the set of logically related actions.
        /// </summary>
        /// <returns></returns>
        public virtual Expression<Func<UserAction, bool>> EquivalenceQuery()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Resolves the equivalent action.
        /// </summary>
        /// <param name="existingAction">The existing action.</param>
        public virtual void ResolveEquivalentAction(UserAction existingAction)
        {
            existingAction.Timestamp = this.Timestamp;
        }


        partial void OnCreated()
        {
            RecommenderStatistics.IncrimentUserActionsCreated();
            if (this.Timestamp == default(DateTime))
            {
                this.Timestamp = DateTime.Now;
            }
        }

        #region IUserAction Members

        #endregion

        #region IPersistentItem Members

        /// <summary>
        /// Clones this instance.
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            object clone = this.MemberwiseClone();
            (clone as UserAction).Key = 0;
            return clone;
        }

        #endregion
    }
}
