﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm
{
    /// <summary>
    /// The interface for data providers to the entity-relation based algorithms.
    /// </summary>
    public interface IEntityRelationDataProvider
    {
        /*
         * Remark: the originally specified Dictionary Access API has been removed 
         * in order to achieve a simplify data access and development.
         * 
         */

        /// <summary>
        /// Gets the entity.
        /// </summary>
        /// <param name="entityType">Type of the entity.</param>
        /// <param name="attributes">The attributes.</param>
        /// <returns></returns>
        IEntityRelationDataReader GetEntity(EntityType entityType, int[] attributes);

        /// <summary>
        /// same as GetEntity but returns all attribute columns
        /// </summary>
        /// <param name="entityType">Type of the entity.</param>
        /// <returns></returns>
        IEntityRelationDataReader GetEntity(EntityType entityType);

        /// <summary>
        /// Gets the relation.
        /// </summary>
        /// <param name="relationType">Type of the relation.</param>
        /// <param name="attributes">The attributes.</param>
        /// <returns></returns>
        IEntityRelationDataReader GetRelation(RelationType relationType, int[] attributes);
        /// <summary>
        /// same as GetRelation but returns all attribute columns
        /// </summary>
        /// <param name="relationType">Type of the relation.</param>
        /// <returns></returns>
        IEntityRelationDataReader GetRelation(RelationType relationType);

        /// <summary>
        /// Registers for relation updates.
        /// </summary>
        /// <param name="relationType">Type of the relation.</param>
        /// <param name="engine">The engine.</param>
        void RegisterForRelationUpdates(RelationType relationType, IEntityRelationEngine engine);
        /// <summary>
        /// Registers for entity updates.
        /// </summary>
        /// <param name="entityType">Type of the entity.</param>
        /// <param name="engine">The engine.</param>
        void RegisterForEntityUpdates(EntityType entityType, IEntityRelationEngine engine);
        /// <summary>
        /// Unregisters for relation updates.
        /// </summary>
        /// <param name="relationType">Type of the relation.</param>
        /// <param name="engine">The engine.</param>
        void UnregisterForRelationUpdates(RelationType relationType, IEntityRelationEngine engine);
        /// <summary>
        /// Unregisters for entity updates.
        /// </summary>
        /// <param name="entityType">Type of the entity.</param>
        /// <param name="engine">The engine.</param>
        void UnregisterForEntityUpdates(EntityType entityType, IEntityRelationEngine engine);
    }
}
