﻿using System;
using System.Collections.Generic;
using System.Text;
using MyMediaProject.RecommenderSystem.Core;

namespace MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm
{
    /// <summary>
    /// Interface for recommendation algorithms which are based on a generalized entity relation paradigm,
    /// used specifically for predicting entities and/or relationships.
    /// </summary>
    /// <remarks>
    /// Entity relationships are a generalized way to look at data.  The "nouns" represent the
    /// entities and the "verb" represents the relationship.
    /// <example>
    /// For example, <see cref="User"/> and <see cref="CatalogItem"/> are entities, and the act of rating represents
    /// a relationship.
    /// </example>
    /// </remarks>
    public interface IEntityRelationPredictor
    {
        /// <summary>
        /// Given an identifier for the relation type (e.g. "rating") and the proper key, the predicted value
        /// for this relation is returned.
        /// </summary>
        /// <param name="relationType">Type of the relation.</param>
        /// <param name="keys">The keys.</param>
        /// <returns></returns>
        double PredictRelation(RelationType relationType, int[] keys);

        /// <summary>
        /// This is the TopN method.
        /// </summary>
        /// <param name="entityType">Type of the entity.</param>
        /// <param name="keys">Combination of other relation's keys. E.g. for predicting top-n items, keys[0] contains the user_id for which top-n ratings should be predicted.</param>
        /// <param name="count">The count.</param>
        /// <returns>An ordered array of keys to that entity.</returns>
        int[] PredictEntity(EntityType entityType, int[] keys, int count);
    }
}
