﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Defines a method to filter our catalog items during a prediction on a <see cref="Flow&lt;TCatalogItemType, TUserActionType&gt;"/>.
    /// </summary>
    /// <remarks>
    /// PreFilters are called before engines in a prediction process. Filters  
    /// decide whether an item is going to be predicted (send through the engines) or not by redefining the query.
    /// </remarks>
    /// <typeparam name="TCatalogItemType">The catalog item type.</typeparam>
    public interface IPreFilter<TCatalogItemType>
    {
        /// <summary>
        /// Defines a new (restricted) query of items that are being send through the flow (<see cref="Flow&lt;TCatalogItemType, TUserActionType&gt;"/>).
        /// </summary>
        /// <param name="query">The query.</param>
        /// <returns>a (redefined) query of items that should be predicted.</returns>
        IQueryable<TCatalogItemType> PreFilter(IQueryable<TCatalogItemType> query);
    }
}
