﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Represents a prediction from an <see cref="Engine&lt;TCatalogItemType, TUserActionType&gt;.Predict"/>.
    /// </summary>
    /// <remarks>
    /// A prediction is the result from <see cref="Engine&lt;TCatalogItemType, TUserActionType&gt;.Predict"/> and contains a <see cref="Rating"/>,
    /// <see cref="Explanation"/> and <see cref="RecommenderId"/> that identifies the catalog item the prediction 
    /// was generated for.
    /// </remarks>
    [Serializable]
    public class Prediction : Prediction<CatalogItem>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Prediction"/> class.
        /// </summary>
        /// <param name="catalogItem">The catalog item.</param>
        public Prediction(CatalogItem catalogItem)
            : base(catalogItem)
        {
        }
    }
    /// <summary>
    /// Represents a prediction from an <see cref="Engine&lt;TCatalogItemType, TUserActionType&gt;.Predict"/>.
    /// </summary>
    /// <remarks>
    /// A prediction is the result from <see cref="Engine&lt;TCatalogItemType, TUserActionType&gt;.Predict"/> and contains a <see cref="Rating"/>,
    /// <see cref="Explanation"/> and <see cref="RecommenderId"/> that identifies the catalog item the prediction 
    /// was generated for.
    /// </remarks>
    /// <typeparam name="TCatalogItemType">The catalog item type.</typeparam>
    [Serializable]
    public class Prediction<TCatalogItemType>
        where TCatalogItemType : CatalogItem
    {
        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <value>The value.</value>
        public double Value
        {
            get
            {
                return this.Rating;
            }
        }
        /// <summary>
        /// Gets or sets the rating.
        /// </summary>
        /// <value>The rating.</value>
        public double Rating { get; set; }
        /// <summary>
        /// Gets or sets the explanation.
        /// </summary>
        /// <value>The explanation.</value>
        public Explanation Explanation { get; set; }
        private TCatalogItemType catalogItem;

        private Prediction() { }

        /// <summary>
        /// Initializes a new instance of the <see cref="Prediction&lt;TCatalogItemType&gt;"/> class.
        /// </summary>
        /// <param name="catalogItem">The catalog item.</param>
        public Prediction(TCatalogItemType catalogItem)
        {
            this.catalogItem = catalogItem;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Prediction&lt;TCatalogItemType&gt;"/> class.
        /// </summary>
        /// <param name="catalogItem">The catalog item.</param>
        /// <param name="rating">The rating.</param>
        public Prediction(TCatalogItemType catalogItem, double rating)
        {
            this.catalogItem = catalogItem;
            this.Rating = rating;
        }

        /// <summary>
        /// Gets the catalog item.
        /// </summary>
        /// <value>The catalog item.</value>
        public TCatalogItemType CatalogItem
        {
            get
            {
                return this.catalogItem;
            }
        }
    }
}
