﻿using System;
using System.Collections.Generic;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Base identity class.
    /// </summary>
    [Serializable]
    public class RecommenderId : IEquatable<RecommenderId>, IEqualityComparer<RecommenderId>
    {
        private Guid value;

        /// <summary>
        /// Initializes a new instance of the <see cref="RecommenderId"/> class.
        /// </summary>
        /// <param name="value">The value.</param>
        public RecommenderId(Guid value)
        {
            this.value = value;
        }

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <value>The value.</value>
        public Guid Value
        {
            get
            {
                return this.value;
            }
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            return this.value.ToString();
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"/>.
        /// </returns>
        public override int GetHashCode()
        {
            return this.value.GetHashCode();
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"/> to compare with the current <see cref="T:System.Object"/>.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            RecommenderId compareObject = obj as RecommenderId;

            if (compareObject != null)
            {
                return value.Equals(compareObject.value);
            }
            else
            {
                return base.Equals(obj);
            }
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="firstValue">The first value.</param>
        /// <param name="secondValue">The second value.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(RecommenderId firstValue, RecommenderId secondValue)
        {
            if (System.Object.ReferenceEquals(firstValue, secondValue))
            {
                return true;
            }

            if (((object)firstValue == null) || ((object)secondValue == null))
            {
                return false;
            }
            
            return firstValue.value == secondValue.value;
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="firstValue">The first value.</param>
        /// <param name="secondValue">The second value.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(RecommenderId firstValue, RecommenderId secondValue)
        {
            return !(firstValue == secondValue);
        }

        #region IEquatable<RecommenderId> Members

        /// <summary>
        /// Indicates whether the current object is equal to another object of the same type.
        /// </summary>
        /// <param name="other">An object to compare with this object.</param>
        /// <returns>
        /// true if the current object is equal to the <paramref name="other"/> parameter; otherwise, false.
        /// </returns>
        public bool Equals(RecommenderId other)
        {
            return this.value.Equals(other.value);
        }

        #endregion

        #region IEqualityComparer<RecommenderId> Members

        /// <summary>
        /// Determines whether the specified objects are equal.
        /// </summary>
        /// <param name="x">The first object of type <paramref name="T"/> to compare.</param>
        /// <param name="y">The second object of type <paramref name="T"/> to compare.</param>
        /// <returns>
        /// true if the specified objects are equal; otherwise, false.
        /// </returns>
        public bool Equals(RecommenderId x, RecommenderId y)
        {
            return x.value.Equals(y.value);
        }

        /// <summary>
        /// Returns a hash code for the specified object.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"/> for which a hash code is to be returned.</param>
        /// <returns>A hash code for the specified object.</returns>
        /// <exception cref="T:System.ArgumentNullException">The type of <paramref name="obj"/> is a reference type and <paramref name="obj"/> is null.</exception>
        public int GetHashCode(RecommenderId obj)
        {
            return obj.value.GetHashCode();
        }

        #endregion
    }
}
