﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Interface for stores which have change tracking.  This allows modifications to the store
    /// to be numbered, and the differences between two versions to be queried.
    /// </summary>
    internal interface ITrackableDataStore
    {
        /// <summary>
        /// Gets the differences.
        /// </summary>
        /// <param name="startChangeNumber">The start change number.</param>
        /// <param name="endChangeNumber">The end change number.</param>
        /// <returns>a list with all changes.</returns>
        /// <remark>
        /// Gets a list of all changes that make up the differences between the two given revisions starting with and including the
        /// change number specified in startChangeNumberand ending with and including the change number specified in endChangeNumber.
        /// All change numbers are ascending integer numbers, starting with 1 for the first change of the store.
        /// The list of changes does not contain every single change, but a list of all changes that have to be taken into
        /// account to get a picture of the diff between the startChangeNumber and the endChangeNumber.
        /// I.e. if in change number 50 an item with the id "a" gets added, and then gets removed in change number 60, the list of changes between 40 and 70 will not show
        /// this change because it does not make a difference for the state of the store.
        /// </remark>
        IList<IDataStoreChange> GetDifferences(int startChangeNumber, int endChangeNumber);
    }
}
