﻿using System;
using System.Collections.Generic;
using System.Data.Linq;
using System.Globalization;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    internal class StoreTracker<TStoreChangeType> where TStoreChangeType : class, IDataStoreChange, new()
    {
        public static void AddChange(Table<TStoreChangeType> changeTable, int changeNumber, StoreChangeType changeType, int changeId)
        {
            TStoreChangeType change = new TStoreChangeType() { ChangeNumber = changeNumber, ChangeType = changeType, ChangedItemId = changeId};
            changeTable.InsertOnSubmit(change);
        }
        public static int ChangeNumber(Table<TStoreChangeType> changeTable)
        {
            int changeNumber = 0;
            
            //This seems to be the most efficient way to check for an empty table.
            if (null != changeTable.FirstOrDefault())
            {
                changeNumber = changeTable.Max(c => c.ChangeNumber);
            }
            return changeNumber;
        }
        public static IList<IDataStoreChange> GetDifferences(Table<TStoreChangeType> changeTable, int startChangeNumber, int endChangeNumber)
        {
            Dictionary<int, IDataStoreChange> result = new Dictionary<int, IDataStoreChange>();
            int i = 0;

            List<TStoreChangeType> changes = changeTable.Where(
                c => c.ChangeNumber >= startChangeNumber && c.ChangeNumber <= endChangeNumber).ToList();
            while (i < changes.Count)
            {
                IDataStoreChange changeToAdd = changes[i];
                if (result.ContainsKey(changeToAdd.ChangedItemId))
                {
                    switch ((StoreChangeType)result[changeToAdd.ChangedItemId].ChangeType)
                    {
                        case StoreChangeType.Added:
                            switch ((StoreChangeType)changeToAdd.ChangeType)
                            {
                                case StoreChangeType.Added:
                                    ThrowOnInvalidChangeSequence(result[changeToAdd.ChangedItemId], changeToAdd);
                                    break;
                                case StoreChangeType.Changed:
                                    break;
                                case StoreChangeType.Removed:
                                    result.Remove(changeToAdd.ChangedItemId);
                                    break;
                            }
                            break;
                        case StoreChangeType.Changed:
                            switch ((StoreChangeType)changeToAdd.ChangeType)
                            {
                                case StoreChangeType.Added:
                                    ThrowOnInvalidChangeSequence(result[changeToAdd.ChangedItemId], changeToAdd);
                                    break;
                                case StoreChangeType.Changed:
                                    break;
                                case StoreChangeType.Removed:
                                    result[changeToAdd.ChangedItemId] = (UserActionStoreChange)changeToAdd;
                                    break;
                            }
                            break;
                        case StoreChangeType.Removed:
                            switch ((StoreChangeType)changeToAdd.ChangeType)
                            {
                                case StoreChangeType.Added:
                                    result[changeToAdd.ChangedItemId] = new TStoreChangeType { ChangeNumber = changeToAdd.ChangeNumber, ChangeType = StoreChangeType.Changed, ChangedItemId = changeToAdd.ChangedItemId };
                                    break;
                                case StoreChangeType.Changed:
                                case StoreChangeType.Removed:
                                    ThrowOnInvalidChangeSequence(result[changeToAdd.ChangedItemId], changeToAdd);
                                    break;
                            }
                            break;
                    }
                }
                else
                {
                    result.Add(changes[i].ChangedItemId, changes[i]);
                }
                i++;
            }
            return new List<IDataStoreChange>(result.Values);
        }
        private static void ThrowOnInvalidChangeSequence(IDataStoreChange firstChange, IDataStoreChange secondChange)
        {
            throw new InvalidOperationException(
                string.Format(
                CultureInfo.InvariantCulture,
                "Invalid change sequence {0}/{1} for changedItemId {2}",
                firstChange.ChangeType,
                secondChange.ChangeType,
                firstChange.ChangedItemId));
        }
    }
}
