﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using System.Globalization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Core.Databases;

namespace RecommenderCoreTest.Databases.CatalogItemStoreTest
{
    /// <summary>
    /// Summary description for CatalogItemTest
    /// </summary>
    [TestClass]
    public class CatalogItemTest
    {
        ProgramId catalogItemTestProgramId = new ProgramId(new Guid("{E35AC5D3-424D-43e6-BBD5-C0DC189FF04B}"),
            "Test store for the CatalogItemTest class.");
        public CatalogItemTest()
        {
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        //[TestMethod]
        public void CatalogItemSimpleExample()
        {
            CatalogItemStore store = new CatalogItemStore(catalogItemTestProgramId);
            CatalogItemStoreDataContext context = store.DataContext;
            {
                if (context.DatabaseExists())
                {
                    context.DeleteDatabase();
                }
                context.CreateDatabase();

                //Simple addition of linq entity (class)
                CatalogItem item = new CatalogItem();
                ItemTitle itemTitle = new ItemTitle();
                Language language = new Language();
                language.Code = System.Globalization.CultureInfo.CurrentCulture.Name;

                itemTitle.Title = "Gone with the wind";
                itemTitle.Language = language;
                item.ItemTitles.Add(itemTitle);
                context.CatalogItems.InsertOnSubmit(item);
                context.SubmitChanges();

                //Keeping FK tables properly normalized; i.e. not having duplicate US English entries
                //The trick is that you have to reference the exact language entry, with the appropriate key, which implies it must be queried or cached locally.
                //Perhaps a static class or a class factory which does this?  Maybe something like this:
                //class Language
                //{
                //    static Language GetLanguage(CatalogItemStoreDataContext context)
                //    {
                //        ...
                //    }
                //}
                //This might be best done with a wrapper/dervied class, which provides modification functionality?
                Language foundLanguage = context.Languages.FirstOrDefault(dbLanguage => dbLanguage.Code == System.Globalization.CultureInfo.CurrentCulture.Name);
                if (null == foundLanguage)
                {
                    //Create a new language entry
                    foundLanguage = new Language();
                    foundLanguage.Code = System.Globalization.CultureInfo.CurrentCulture.Name;
                }
                item = new CatalogItem();
                itemTitle = new ItemTitle();

                itemTitle.Title = "Gone with the wind 2";
                itemTitle.Language = foundLanguage;
                item.ItemTitles.Add(itemTitle);
                context.CatalogItems.InsertOnSubmit(item);
                context.SubmitChanges();
            }

            {
                CatalogItem item = context.CatalogItems.FirstOrDefault(c => c.ItemTitles.First().Title == "Gone with the wind 2");

                //When this is all done, there should be two items in the DB, two ItemTitles in the DB, and one Language.
                Assert.AreEqual(2, context.CatalogItems.Count());
                Assert.AreEqual(2, context.ItemTitles.Count());
                Assert.AreEqual(1, context.Languages.Count());
            }
        }

        //[TestMethod]
        public void CatalogItemKeyword()
        {
            CatalogItemStore store = new CatalogItemStore(catalogItemTestProgramId);
            CatalogItemStoreDataContext context = store.DataContext;
            {
                if (context.DatabaseExists())
                {
                    context.DeleteDatabase();
                }
                context.CreateDatabase();
                CatalogItem item = new CatalogItem();

                //One way to add a keyword
                item.AddKeyword(context, context.CreateKeyword("Fun!", CultureInfo.CurrentCulture), 0);

                //Another way to add a keyword
                item.AddKeyword(context, "MoreFun!", CultureInfo.CurrentCulture, 0);

                context.CatalogItems.InsertOnSubmit(item);
                context.SubmitChanges();
            }
            {
                CatalogItem item = context.CatalogItems.First();
                item.AddKeyword(context, "MoreFun!", CultureInfo.CurrentCulture, 0);
            }
        }

        [TestMethod]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void CatalogStoreComplexItemCreation()
        {
            using (CatalogItemStore store = new CatalogItemStore(catalogItemTestProgramId))
            {
                CatalogItemStoreDataContext context = store.DataContext;
                {
                    int expectedItemsCreatedCount = RecommenderStatistics.CatalogItemsCreated;
                    if (context.DatabaseExists())
                    {
                        context.DeleteDatabase();
                    }
                    context.CreateDatabase();
                    CatalogItem item1 = context.CreateItem();
                    expectedItemsCreatedCount++;

                    CatalogItem item2 = context.CreateItem();
                    expectedItemsCreatedCount++;

                    CultureInfo englishCultureInfo = new CultureInfo("en");
                    CultureInfo germanCultureInfo = new CultureInfo("de");
                    string title1 = "The Boat";
                    string title2 = "Das Boot";
                    string keyword1 = "boat";
                    string keyword2 = "boot";
                    string keyword5 = "SharedKeyword";
                    string genre1 = "action";
                    string uri1 = "foo";
                    string description1 = "Some description in english.";
                    string description2 = "Some description in german.";
                    string propName1 = "PropertyName";
                    string propValue1 = "PropertyValue";
                    item1.AddTitle(context, title1, englishCultureInfo);
                    item1.AddKeyword(context, keyword1, englishCultureInfo, 0);
                    item1.AddDescription(context, description1, englishCultureInfo);
                    item1.AddCaptionLanguage(context, englishCultureInfo);
                    item1.AddAudioLanguage(context, englishCultureInfo, false);

                    item1.AddTitle(context, title2, germanCultureInfo);
                    item1.AddKeyword(context, keyword2, germanCultureInfo, 0);
                    item1.AddDescription(context, description2, germanCultureInfo);
                    item1.AddCaptionLanguage(context, germanCultureInfo);
                    item1.AddAudioLanguage(context, germanCultureInfo, false);

                    item1.AddKeyword(context, keyword5, englishCultureInfo, 0);
                    item1.AddProperty(context, propName1, propValue1);

                    item1.AddGenre(context, genre1, string.Empty);
                    item1.AddLocator(context, uri1);

                    item1.AddRelationship(context, "Relationship Name", "A URI", "A URL");

                    string title3 = "Hello";
                    string title4 = "Hallo";
                    string genre2 = "romance";
                    string keyword3 = "sweet";
                    string keyword4 = "suss";
                    string uri2 = "bar";
                    string description3 = "Another english decription.";
                    string description4 = "Another german description.";
                    item2.AddTitle(context, title3, englishCultureInfo);
                    item2.AddKeyword(context, keyword3, englishCultureInfo, 0);
                    item2.AddDescription(context, description3, englishCultureInfo);
                    item2.AddCaptionLanguage(context, englishCultureInfo);

                    item2.AddTitle(context, title4, germanCultureInfo);
                    item2.AddKeyword(context, keyword4, germanCultureInfo, 0);
                    item2.AddDescription(context, description4, germanCultureInfo);
                    item2.AddCaptionLanguage(context, germanCultureInfo);

                    item2.AddKeyword(context, keyword5, englishCultureInfo, 0);
                    item2.AddProperty(context, propName1, propValue1);

                    item2.AddGenre(context, genre2, string.Empty);
                    item2.AddLocator(context, uri2);

                    context.SubmitChanges();

                    CatalogItem item3 = context.CreateItem();
                    expectedItemsCreatedCount++;
                    item3.IsGroup = true;
                    item3.IsOrdered = true;

                    item1.AddToGroup(context, item3);
                    expectedItemsCreatedCount++; //Associating two items touches the related item as well.

                    item2.AddToGroup(context, item3);
                    expectedItemsCreatedCount++; //Associating two items touches the related item as well.

                    context.SubmitChanges();

                    Assert.AreEqual<int>(3, context.CatalogItems.Count());
                    Assert.AreEqual<int>(4, context.ItemTitles.Count());
                    Assert.AreEqual<int>(6, context.ItemKeywords.Count());
                    Assert.AreEqual<int>(5, context.Keywords.Count());
                    Assert.AreEqual<int>(3, context.Genres.Count());
                    Assert.AreEqual<int>(2, context.Locators.Count());
                    Assert.AreEqual<int>(4, context.ItemDescriptions.Count());
                    Assert.AreEqual<int>(2, context.CaptionLanguages.Count());
                    Assert.AreEqual<int>(2, context.ItemAudioLanguages.Count());
                    Assert.AreEqual<int>(2, context.ItemProperties.Count());
                    Assert.AreEqual<int>(1, context.Properties.Count());
                    Assert.AreEqual<int>(1, context.Relationships.Count());
                    Assert.AreEqual<int>(1, context.ItemRelatedMaterials.Count());
                    Assert.AreEqual<int>(1, context.Relationships.Count());
                    Assert.AreEqual<int>(2, context.ItemGroupMembers.Count());

                    IQueryable<CatalogItem> query = from element in context.ItemTitles
                                                    where element.Title == title1
                                                    select element.CatalogItem;
                    List<CatalogItem> queryResult = query.ToList();
                    expectedItemsCreatedCount++;
                    query = context.CatalogItems.Where(element => element.ItemTitles.Any(elementTitle => elementTitle.Title == title2));
                    queryResult = query.ToList();
                    expectedItemsCreatedCount++;
                    Assert.AreEqual<int>(expectedItemsCreatedCount, RecommenderStatistics.CatalogItemsCreated,
                        "Query of items by title executes on server.");

                    query = context.CatalogItems.Where(e => e.ItemKeywords.Any(keyword => keyword.Keyword.Word == keyword1));
                    queryResult = query.ToList();
                    expectedItemsCreatedCount++;
                    Assert.AreEqual<int>(expectedItemsCreatedCount, RecommenderStatistics.CatalogItemsCreated,
                        "Query of items by keyword executes on server.");

                    query = context.CatalogItems.Where(element => element.ItemGenres.Any(elementGenre => elementGenre.Genre.Name == genre1));
                    queryResult = query.ToList();
                    expectedItemsCreatedCount++;
                    Assert.AreEqual<int>(expectedItemsCreatedCount, RecommenderStatistics.CatalogItemsCreated,
                        "Query of items by genre executes on server.");

                    //Duplicate all associations
                    item1.AddTitle(context, title1, new CultureInfo("en"));
                    item1.AddKeyword(context, keyword1, new CultureInfo("en"), 0);
                    item1.AddTitle(context, title2, new CultureInfo("de"));
                    item1.AddKeyword(context, keyword2, new CultureInfo("de"), 0);
                    item1.AddGenre(context, genre1, string.Empty);
                    item1.AddLocator(context, uri1);
                    item2.AddTitle(context, title3, new CultureInfo("en"));
                    item2.AddKeyword(context, title3, new CultureInfo("en"), 0);
                    item2.AddTitle(context, title4, new CultureInfo("de"));
                    item2.AddKeyword(context, title4, new CultureInfo("de"), 0);
                    item2.AddGenre(context, genre2, string.Empty);
                    item2.AddLocator(context, uri2);

                    //Expect no overall change
                    Assert.AreEqual<int>(3, context.CatalogItems.Count());
                    Assert.AreEqual<int>(4, context.ItemTitles.Count());
                    Assert.AreEqual<int>(6, context.ItemKeywords.Count());
                    Assert.AreEqual<int>(5, context.Keywords.Count());
                    Assert.AreEqual<int>(3, context.Genres.Count());
                    Assert.AreEqual<int>(2, context.Locators.Count());
                    Assert.AreEqual<int>(4, context.ItemDescriptions.Count());
                    Assert.AreEqual<int>(2, context.CaptionLanguages.Count());
                    Assert.AreEqual<int>(2, context.ItemAudioLanguages.Count());
                    Assert.AreEqual<int>(2, context.ItemProperties.Count());
                    Assert.AreEqual<int>(1, context.Properties.Count());
                    Assert.AreEqual<int>(1, context.Relationships.Count());
                    Assert.AreEqual<int>(1, context.ItemRelatedMaterials.Count());
                    Assert.AreEqual<int>(1, context.Relationships.Count());
                    Assert.AreEqual<int>(2, context.ItemGroupMembers.Count());
                }
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }        
    }
}
