﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyMediaProject.RecommenderSystem.Core.Eventing;

namespace MyMediaProject.RecommenderSystem.Core.Eventing
{
    class TestEvent1 : RecommenderEvent { }
    class TestEvent2 : RecommenderEvent { }
    class TestEventArgs : RecommenderEventArgs { }

    /// <summary>
    /// Summary description for EventingTest
    /// </summary>
    [TestClass]
    public class EventTest
    {
        private int recommenderCallCount;
        private int testCallCount;

        private void RecommenderEventHandler(EventArgs args)
        {
            recommenderCallCount++;
        }

        private void TestEventHandler(EventArgs args)
        {
            testCallCount++;
        }

        [TestInitialize()]
        public void Initialize()
        {
            recommenderCallCount = 0;
            testCallCount = 0;
        }

        [TestCleanup()]
        public void Cleanup()
        {
            new TestEvent1().Unsubscribe(this.RecommenderEventHandler);
            new TestEvent1().Unsubscribe(this.TestEventHandler);
        }

        [TestMethod]
        public void EventTestFirstEvent()
        {
            new TestEvent1().Subscribe(this.RecommenderEventHandler);
            new TestEvent2().Subscribe(this.TestEventHandler);

            new TestEvent1().Fire(new TestEventArgs());

            Assert.IsTrue(1 <= recommenderCallCount);
            Assert.AreEqual<int>(0, testCallCount);
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        public void EventTestSecondEvent()
        {
            new TestEvent1().Subscribe(this.RecommenderEventHandler);
            new TestEvent2().Subscribe(this.TestEventHandler);

            new TestEvent2().Fire(new TestEventArgs());

            Assert.IsTrue(0 <= recommenderCallCount);
            Assert.AreEqual<int>(1, testCallCount);
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        public void EventTestFirstEventSubscriptionOnly()
        {
            new TestEvent1().Subscribe(this.RecommenderEventHandler);

            new TestEvent1().Fire(new TestEventArgs());

            Assert.IsTrue(1 <= recommenderCallCount);
            Assert.AreEqual<int>(0, testCallCount);
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        public void EventTestSecondEventSubscriptionOnly()
        {
            new TestEvent2().Subscribe(this.TestEventHandler);

            new TestEvent2().Fire(new TestEventArgs());

            Assert.IsTrue(0 <= recommenderCallCount);
            Assert.AreEqual<int>(1, testCallCount);
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        public void EventTestNoSubscribers()
        {
            new TestEvent1().Fire(new TestEventArgs());
            new TestEvent2().Fire(new TestEventArgs());

            Assert.IsTrue(0 <= recommenderCallCount);
            Assert.AreEqual<int>(0, testCallCount);
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }
    }
}
