﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;

using Microsoft.VisualStudio.TestTools.UnitTesting;

using MyMediaProject.RecommenderSystem.Core;

namespace RecommenderCoreTest
{
    /// <summary>
    /// Summary description for FlowTest
    /// </summary>
    [TestClass]
    public class FlowTest
    {
        public FlowTest()
        {
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void FlowTestEmptyObject()
        {
            FlowId fid = new FlowId();
            Flow<CatalogItem, UserAction> flow = new Flow<CatalogItem, UserAction>(fid);

            Assert.IsTrue(flow.Engines.Count == 0, "number of engines on empty object is not 0");
            Assert.IsTrue(flow.PreFilters.Count == 0, "number of pre-filters on empty object is not 0");
            Assert.IsTrue(flow.PostFilters.Count == 0, "number of post-filters on empty object is not 0");
            Assert.IsTrue(flow.Hybridizer != null, "default hybridizer is NULL");
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        public void FlowTestWithCancelPreFilter()
        {
            FlowId fid = new FlowId();
            Flow<CatalogItem, UserAction> flow = new Flow<CatalogItem, UserAction>(fid);

            flow.AddPreFilter(new FlowTestCancelPreFilter<CatalogItem>());
            flow.AddEngine(new FlowTestFixedRatingEngine<CatalogItem, UserAction>(1.0));
            flow.Hybridizer = new FlowTestAveragingHybridizer<CatalogItem>();

            Guid userGuid = Guid.NewGuid();
            User uid = new User() { UserName = userGuid.ToString()};
            IQueryable<CatalogItem> tmp = new List<CatalogItem>() { new CatalogItem() }.AsQueryable();

            IList<Prediction<CatalogItem>> result = flow.Process(tmp, uid);

            Assert.IsTrue(result != null, "Empty flow was not expected.");
            Assert.IsTrue(result.Count == 0, "Pre-Filter did not work.");
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        public void FlowTestWithCancelPostFilter()
        {
            FlowId fid = new FlowId();
            Flow<CatalogItem, UserAction> flow = new Flow<CatalogItem, UserAction>(fid);

            flow.AddEngine(new FlowTestFixedRatingEngine<CatalogItem, UserAction>(1.0));
            flow.Hybridizer = new FlowTestAveragingHybridizer<CatalogItem>();
            flow.AddPostFilter(new FlowTestCancelPostFilter<CatalogItem>());

            Guid userGuid = Guid.NewGuid();
            User uid = new User() { UserName = userGuid.ToString()};
            IQueryable<CatalogItem> tmp = new List<CatalogItem>() { new CatalogItem() }.AsQueryable();

            IList<Prediction<CatalogItem>> result = flow.Process(tmp, uid);

            Assert.IsTrue(result != null, "Empty flow was not expected.");
            Assert.IsTrue(result.Count == 0, "Post-Filter did not work.");
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }
        
        [TestMethod]
        public void FlowTestWithAveragingHybridizer()
        {
            FlowId fid = new FlowId();
            Flow<CatalogItem, UserAction> flow = new Flow<CatalogItem, UserAction>(fid);

            flow.AddEngine(new FlowTestFixedRatingEngine<CatalogItem, UserAction>(1.0));
            flow.Hybridizer = new FlowTestAveragingHybridizer<CatalogItem>();

            Guid userGuid = Guid.NewGuid();
            User uid = new User() { UserName = userGuid.ToString()};
            IQueryable<CatalogItem> tmp = new List<CatalogItem>() { new CatalogItem() }.AsQueryable();


            IList<Prediction<CatalogItem>> result = flow.Process(tmp, uid);

            Assert.IsTrue(result != null, "Empty flow was not expected.");
            Assert.IsTrue(result.Count == 1 , "Only 1 result was expected.");
            Assert.IsTrue(result[0].Rating == 1.0, "Flow not completed.");
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }


        public void FlowTestWithAveragingHybridizerMultipleEngines()
        {
            FlowId fid = new FlowId();
            Flow<CatalogItem, UserAction> flow = new Flow<CatalogItem, UserAction>(fid);

            flow.AddEngine(new FlowTestFixedRatingEngine<CatalogItem, UserAction>(0.2));
            flow.AddEngine(new FlowTestFixedRatingEngine<CatalogItem, UserAction>(0.4));
            flow.Hybridizer = new FlowTestAveragingHybridizer<CatalogItem>();

            Guid userGuid = Guid.NewGuid();
            User uid = new User() { UserName = userGuid.ToString()};
            IQueryable<CatalogItem> tmp = new List<CatalogItem>() { new CatalogItem() }.AsQueryable();

            IList<Prediction<CatalogItem>> result = flow.Process(tmp, uid);

            Assert.IsTrue(result != null, "Empty flow was not expected.");
            Assert.IsTrue(result.Count == 1, "Only 1 result was expected.");
            Assert.IsTrue(result[0].Rating == 0.3, "Flow not completed.");
        }
    }
}
