using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Core.Config;
using MyMediaProject.RecommenderSystem.Core.Databases;
using System.Collections.ObjectModel;
using System.IO;
using System.Globalization;

namespace RecommenderCoreTest
{
    [TestClass]
    public class ItemStoreTest
    {
        private CatalogItemStore  itemStore;
        private List<int> eventedChangeNumbers;
        private StoreChangeEvent storeChangeEvent;
        private RecommenderEventHandler eventHandler;
        public const string TestStoreId = "{9891E701-2C49-440f-AC11-315A2CA436F5}";
        public const string CopyStoreId = "{5562878B-E5B9-491b-BD85-57CDA600C434}";
        public const string TestApplicationId = "{015D2DA1-B843-44e7-8A0D-A1A87109BD22}";
        public const string TestApplicationFriendlyName = "ItemTest";

        [ClassInitialize]
        static public void ClassInitialize(TestContext testContext)
        {
            ConfigManager.Initialize(true);
            ItemStoreTest.ClassCleanUp();
        }
        
        [ClassCleanup]
        static public void ClassCleanUp()
        {
            CatalogItemStore itemStore = new CatalogItemStore(
                new ProgramId(new Guid(TestApplicationId), TestApplicationFriendlyName));
            itemStore.DeleteDatabase();
        }

        public ItemStoreTest()
        {
        }

        [TestInitialize]
        public void Initialize()
        {
            eventedChangeNumbers = new List<int>();
            itemStore = new CatalogItemStore(
                new ProgramId(new Guid(TestApplicationId), TestApplicationFriendlyName));

            itemStore.Initialize();

            storeChangeEvent = new StoreChangeEvent();
            this.eventHandler = new RecommenderEventHandler(StoreChanged);
            storeChangeEvent.Subscribe(this.eventHandler);
        }

        [TestCleanup]
        public void TestCleanup()
        {
            this.itemStore.RemoveAll();
            this.Uninitialize();
        }

        public void Uninitialize()
        {
            this.itemStore.Dispose();
            this.storeChangeEvent.Unsubscribe(this.eventHandler);
        }

        void StoreChanged(EventArgs e)
        {
            if (e is StoreChangedEventArgs)
            {
                StoreChangedEventArgs eventArgs = e as StoreChangedEventArgs;
                eventedChangeNumbers.Add(eventArgs.NewChangeNumber);
            }
        }

        private void AssertChange(ICollection<IStoreChange> changes,  StoreChangeType changeType, RecommenderId itemId)
        {
            bool seenOnce = false;

            foreach (IStoreChange change in changes)
            {
                if (change.ChangedItemId == itemId)
                {
                    if (seenOnce)
                    {
                        Assert.Fail(string.Format("More than one change for changedItemId {0}: change type: {1}, change number: {2}", new object[] { itemId, change.ChangeType, change.ChangeNumber }));
                    }
                    else
                    {
                        Assert.IsTrue(change.ChangeType == changeType);
                        seenOnce = true;
                    }
                }
            }
            Assert.IsTrue(seenOnce, string.Format("There is no change for the changedItemId {0}", itemId));
        }

        [TestMethod]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void ItemStoreTestHistory()
        {
            CatalogItemStoreDataContext dataContext = this.itemStore.DataContext;
            CatalogItem item = dataContext.CreateItem();
            item.AddTitle(dataContext, "Terminator 2", CultureInfo.CurrentCulture);
            dataContext.SubmitChanges();

            item = null;

            item = this.itemStore.FirstOrDefault();
            Assert.IsTrue(item != null);
        }

    }
}
