﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using EMIC.Design;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Framework;


namespace MyMediaProject.RecommenderSystem.Framework.Metrics
{
    /// <summary>
    /// Partitions the user actions into a training and test set, where the
    /// test set is made of N random ratings.
    /// </summary>
    /// <typeparam name="TUserActionType">The type of the user action type.</typeparam>
    public class RandomAllButNTestSet<TUserActionType> : AllButNTestSetBase<TUserActionType>
        where TUserActionType : RatingAction
    {
        private Random randomIndexGenerator;

        /// <summary>
        /// Initializes a new instance of the <see cref="RandomAllButNTestSet&lt;TUserActionType&gt;"/> class.
        /// </summary>
        /// <param name="recommender">The recommender.</param>
        /// <param name="testSetSizeN">The test set size N.</param>
        public RandomAllButNTestSet(Recommender recommender, int testSetSizeN)
            : base(recommender, testSetSizeN)
        {
            this.randomIndexGenerator = new Random();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RandomAllButNTestSet&lt;TUserActionType&gt;"/> class.
        /// </summary>
        /// <param name="recommender">The recommender.</param>
        /// <param name="testSetSizeN">The test set size N.</param>
        /// <param name="randomNumberSeed">The random number seed.</param>
        public RandomAllButNTestSet(Recommender recommender, int testSetSizeN, int randomNumberSeed)
            : base(recommender, testSetSizeN)
        {
            this.randomIndexGenerator = new Random(randomNumberSeed);
        }

        /// <summary>
        /// Gets the training set from user feedback.
        /// </summary>
        /// <param name="feedbackItems">The feedback items.</param>
        /// <param name="toRemove">To remove.</param>
        /// <param name="count">The count.</param>
        protected override void GetTrainingSetFromUserFeedback(System.Collections.ObjectModel.Collection<TUserActionType> feedbackItems, System.Collections.ObjectModel.Collection<TUserActionType> toRemove, int count)
        {
            for (int i = 0; i < count; i++)
            {
                int randomUserFeedbackIndex = this.randomIndexGenerator.Next(feedbackItems.Count());
                TUserActionType testFeedback = feedbackItems.ElementAt<TUserActionType>(
                    randomUserFeedbackIndex);
                toRemove.Add(testFeedback);
            }
        }
    }
}
