﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Core;
using EMIC.Design;

namespace MyMediaProject.RecommenderSystem.Framework
{
    /// <summary>
    /// A threadsafe singleton template for the <see cref="Recommender"/>.
    /// </summary>
    public sealed class RecommenderSingleton<TRecommenderType> where TRecommenderType : Recommender, new()
    {
        private static volatile TRecommenderType instance;
        private static object syncRoot = new object();

        private RecommenderSingleton(){}

        /// <summary>
        /// Gets the singleton.  This can be done asyncronously, and all instance calls will block until the 
        /// recommender is initialized.
        /// </summary>
        /// <value>The instance.</value>
        public static TRecommenderType Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            TraceLog.Write(LogEventType.Information, "Initializing Recommender Singleton.");
                            TRecommenderType newInstance = new TRecommenderType();
                            newInstance.Initialize();
                            RecommenderSingleton<TRecommenderType>.instance = newInstance;
                        }
                    }
                }

                return instance;
            }
        }
    }
}
