﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm;
using MyMediaProject.RecommenderSystem.Algorithms;
using MyMediaProject.RecommenderSystem.Algorithms.Utils;

namespace EngineTest
{
    /// <summary>
    /// Test program for recommender algorithms.
    /// </summary>
    /// <author>Steffen Rendle, University of Hildesheim</author>
    class Program
    {
        static void Main(string[] args)
        {
            System.Console.WriteLine("Testing the recommender implementations\n");
            WP2Backend.GetInstance();

            // Rating prediction
            System.Console.WriteLine("\nRating prediction:");
            DataSetMemoryBased test_rated = new DataSetMemoryBased(new DataSetMemoryBased.DataType[] { DataSetMemoryBased.DataType.dt_int, DataSetMemoryBased.DataType.dt_int, DataSetMemoryBased.DataType.dt_double }, "..\\..\\..\\Data\\ml1m-test.txt");

            // Global average...  
            System.Console.Write("Global Avg...   ");
            RatingPredictorGlobalAverage ga = new RatingPredictorGlobalAverage();
            ga.EntityRelationDataProvider = WP2Backend.GetInstance();
            ga.MinRatingValue = 1;
            ga.MaxRatingValue = 5;
            ga.Train();
            Evaluate.evaluateRated(ga, test_rated);

            // User average...  
            System.Console.Write("User Avg...     ");
            RatingPredictorUserAverage ua = new RatingPredictorUserAverage();
            ua.EntityRelationDataProvider = WP2Backend.GetInstance();
            ua.MinRatingValue = 1;
            ua.MaxRatingValue = 5;
            ua.Train();
            Evaluate.evaluateRated(ua, test_rated);

            // Item average...  
            System.Console.Write("Item Avg...     ");
            RatingPredictorItemAverage ia = new RatingPredictorItemAverage();
            ia.EntityRelationDataProvider = WP2Backend.GetInstance();
            ia.MinRatingValue = 1;
            ia.MaxRatingValue = 5;
            ia.Train();
            Evaluate.evaluateRated(ia, test_rated);

            // Matrix factorization...  
            System.Console.Write("MF...           ");
            MatrixFactorization mf = new MatrixFactorization();
            mf.EntityRelationDataProvider = WP2Backend.GetInstance();
            mf.MinRatingValue = 1;
            mf.MaxRatingValue = 5;
            mf.Train();
            Evaluate.evaluateRated(mf, test_rated);


            // Item Recommender test
            System.Console.WriteLine("\nItem recommendation:");
            DataSetMemoryBased test_item_rec = new DataSetMemoryBased(new DataSetMemoryBased.DataType[] { DataSetMemoryBased.DataType.dt_int, DataSetMemoryBased.DataType.dt_int, DataSetMemoryBased.DataType.dt_double }, "..\\..\\..\\Data\\ml1m-test.txt");

            // Most popular:            
            System.Console.Write("Most popular... ");
            ItemRecommenderMostPopular mp = new ItemRecommenderMostPopular(RelationType.Viewed);
            mp.EntityRelationDataProvider = WP2Backend.GetInstance();
            mp.Train();
            Evaluate.evaluateItemRecommender(mp, test_item_rec, WP2Backend.GetInstance().GetRelation(RelationType.Viewed), WP2Backend.GetInstance().GetEntity(EntityType.CatalogItem));

            // KNN:            
            System.Console.Write("kNN...          ");
            ItemRecommenderkNN kNN = new ItemRecommenderkNN(RelationType.Viewed);
            kNN.EntityRelationDataProvider = WP2Backend.GetInstance();
            kNN.Train();
            Evaluate.evaluateItemRecommender(kNN, test_item_rec, WP2Backend.GetInstance().GetRelation(RelationType.Viewed), WP2Backend.GetInstance().GetEntity(EntityType.CatalogItem));

            // SVD 8:
            System.Console.Write("SVD...         ");
            ItemRecommenderSVD svd = new ItemRecommenderSVD(RelationType.Viewed);
            svd.EntityRelationDataProvider = WP2Backend.GetInstance();
            svd.num_iter = 8;
            svd.num_feature = 8;
            svd.Train();
            Evaluate.evaluateItemRecommender(svd, test_item_rec, WP2Backend.GetInstance().GetRelation(RelationType.Viewed), WP2Backend.GetInstance().GetEntity(EntityType.CatalogItem));

            // WRMF 8:
            System.Console.Write("WRMF...         ");
            ItemRecommenderWRMF wrmf = new ItemRecommenderWRMF(RelationType.Viewed);
            wrmf.EntityRelationDataProvider = WP2Backend.GetInstance();
            wrmf.num_iter = 8;
            wrmf.num_feature = 8;
            wrmf.init_f_mean = 0.0;
            wrmf.init_f_stdev = 0.1;
            wrmf.regularization = 0;
            wrmf.c_pos = 16;
            wrmf.Train();
            Evaluate.evaluateItemRecommender(wrmf, test_item_rec, WP2Backend.GetInstance().GetRelation(RelationType.Viewed), WP2Backend.GetInstance().GetEntity(EntityType.CatalogItem));

            // Virtual Profile test
            /*
            VirtualProfileRecommender vpr = new VirtualProfileRecommender(WP2Backend.GetInstance());
            System.Console.WriteLine("r(0,0)=" + vpr.PredictRelation(RelationType.VirtualProfileRated, new int[] { 0, 0 }));
            System.Console.WriteLine("r(0,1)=" + vpr.PredictRelation(RelationType.VirtualProfileRated, new int[] { 0, 1 }));
            System.Console.WriteLine("r(100,1)=" + vpr.PredictRelation(RelationType.VirtualProfileRated, new int[] { 100, 1 }));
            System.Console.WriteLine("r(0,100)=" + vpr.PredictRelation(RelationType.VirtualProfileRated, new int[] { 0, 100 }));
            System.Console.WriteLine("r(100,100)=" + vpr.PredictRelation(RelationType.VirtualProfileRated, new int[] { 100, 100 }));
            */


            System.Console.WriteLine("\nDone. Press ENTER to exit.");
            System.Console.ReadLine();

        }
    }
}
