// This is the main DLL file.

#include <stdio.h>
#include "UIMAWrapper.h"

#include "uima/dirwalk.hpp"
#include "uima/filename.hpp"
#include "uima/xmlwriter.hpp"

using namespace uima;
using namespace System;
using namespace System::Collections::Generic;

namespace mymedia {

    UIMAWrapper::UIMAWrapper(String^ descriptor){
        errorInfo = new ErrorInfo();
        (void) ResourceManager::createInstance("MyMedia keyword extractor");
        ResourceManager::getInstance().setLoggingLevel(LogStream::EnMessage);

        annotations = gcnew List<AnnotationType^>();

        
        pEngine = Framework::createAnalysisEngine(true, *stringToUnicode(descriptor), *errorInfo);
        if (pEngine) {
            tcas = pEngine->newCAS();
        } else {
            cout << "Error ID: " << AnalysisEngine::getErrorIdAsCString(errorInfo->getErrorId()) << endl;
            cout << "Message: " << errorInfo->getMessage() << endl;
        }
        cout << "UIMA engine created" << endl;
    }

    UIMAWrapper::~UIMAWrapper() {
        utErrorId = pEngine->destroy();

        delete tcas;
        delete pEngine;
        delete errorInfo;
    }

    UIMAWrapper::!UIMAWrapper() {

    }

    LinkedList<AnnotationWrapper^>^ UIMAWrapper::tag(System::String^ corpus) {
        LinkedList<AnnotationWrapper^>^ resultSet = gcnew LinkedList<AnnotationWrapper^>();

        if(tcas) {
            tcas->reset();
            resultSet = processInternal(stringToUnicode(corpus), pEngine, tcas);
        } else {
            cout << "Can't process due to previous error." << endl;
        }

        return resultSet;
    } 

    void UIMAWrapper::addAnnotationType(AnnotationType^ annotation) {
        annotations->Add(annotation);        
    }

    LinkedList<AnnotationWrapper^>^ UIMAWrapper::processInternal(UnicodeString * ucs, AnalysisEngine * pEngine, CAS* tcas) {
       
        LinkedList<AnnotationWrapper^>^ resultSet = gcnew LinkedList<AnnotationWrapper^>();

        try {
            UnicodeStringRef * ucsr = new UnicodeStringRef(*ucs);
            tcas->setDocumentText(*ucsr);
            /* process the CAS */
            pEngine->process(*tcas);
            
            TypeSystem * typeSystem = Framework::createTypeSystem(pEngine->getAnalysisEngineMetaData(), *errorInfo);
           

            for (int i = 0; i < annotations->Count; i++)
            {
                AnnotationType^ annotation = annotations[i];
                uima::Type type = typeSystem->getType(*(stringToUnicode(annotation->getType())));
                ANIndex index = tcas->getAnnotationIndex(type);
                ANIterator iter = index.iterator();
                if (iter.isValid()) {
                    while (iter.peekNext().isValid()) {
                        AnnotationFS a = iter.get();
                        UnicodeStringRef value;
                        if (annotation->getField()->Equals("text")) {
                            value = a.getCoveredText();
                        } else {
                            Feature feature = type.getFeatureByBaseName(*(stringToUnicode(annotation->getField())));
                            value = a.getStringValue(feature);
                        }
                        
                        resultSet->AddLast(gcnew AnnotationWrapper(gcnew String(value.getBuffer(), 0, value.length()), annotation->getKeywordType()));
                        iter.moveToNext();
                    }
                }
           }
           
           delete ucsr;
           delete ucs;
           delete typeSystem;
           
        } catch (uima::Exception e) {
            cout << "UIMAWrapper: " << e << endl;
        }
        return resultSet;
    }

    void UIMAWrapper::finish() {
        utErrorId = pEngine->destroy();
        //utErrorId = pEngine->batchProcessComplete();
        //if (utErrorId != UIMA_ERR_NONE) {
        //    cout << "Error ID: " << AnalysisEngine::getErrorIdAsCString(utErrorId) << endl;
        //    cout << "Message: " << errorInfo->getMessage() << endl;
        //}        

    }

    UnicodeString* UIMAWrapper::stringToUnicode(String^ string)
    {
        array<wchar_t>^ a = string->ToCharArray();
        UChar * pBuf = (UChar*) malloc(sizeof(UChar) * string->Length);
        for(int i = 0; i < string->Length; i++) {
            pBuf[i] = a[i];
        }

        return new UnicodeString(pBuf, string->Length);
    }
}
