package nl.telin.mymedia.apolda;

//import java.io.InputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import nl.telin.mymedia.uima.annotation.OntologyAnnotation;

import org.apache.uima.analysis_engine.annotator.AnnotatorProcessException;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.jcas.JCas;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

/*
 * Korte uitleg...
 */
public class Apolda {
	
	
	
	private static final String DOCPROP_NROFTOK = "Number of tokens";
	private static final String DOCPROP_NROFCONCEPTS = "Number of recognized concepts";
	private static final String STATUS_STARTDOC = "Doing ontology lookup in";
	private static final String STATUS_ENDDOC = "Ontology lookup complete!";
	private static final String STATUS_STARTREADONT = "Reading ontology";
	private static final String STATUS_ENDREADONT = "term representations read";
	
	private Locale language = null;
	
	private Property prefRepresentationAP = null;
	private Property altRepresentationAP = null;
	private InfModel ontology = null;
	private String sURI = null;

	/**
	 * The initial state of the FSM that backs this gazetteer
	 */
	protected FSMState initialState;

	/**
	 * A set containing all the states of the FSM backing the gazetteer
	 */
	protected Set<FSMState> fsmStates;
	
	public Apolda() {
		
	}

	public boolean initialize(String ontologyURI, String prefRepresentation, String altRepresentation, String sLanguage) {
		boolean success = true;
		
		sURI = ontologyURI;
		
		language = new Locale(sLanguage);
		
		fsmStates = new HashSet<FSMState>();
		
		OntModel model = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM ); 
		if(model == null)
			return false;
		
		try {
			//model.read("file://" + ontologyURI, "RDF/XML"); //TODO 2e argument als parameter (?); 1e argument netter oplossen
			BufferedReader reader = new BufferedReader(new FileReader(ontologyURI));
			model.read(reader, "RDF/XML");
			
			ontology = ModelFactory.createRDFSModel(model);
			
			if(prefRepresentation != null && !prefRepresentation.isEmpty()) {
				 if(!prefRepresentation.contains("#")) { //TODO een beetje erg bottte manier om te testen of het een geldige URI is
					String prefix = ontology.getNsPrefixURI("");
					prefRepresentation = prefix + prefRepresentation;
				}
				prefRepresentationAP = ontology.createProperty(prefRepresentation);
			}
			if(altRepresentation != null && !altRepresentation.isEmpty()) {
				if(!altRepresentation.contains("#")) {
					String prefix = ontology.getNsPrefixURI("");
					altRepresentation = prefix + altRepresentation;
				}
				altRepresentationAP = ontology.createProperty(altRepresentation);
			}
			
			initialState = new FSMState();
			fsmStates.add(initialState);
			
			readOntology();
			
			return success;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return false;
	}
	
	private int readOntology() {

		int nrOfStrings = 0;
		
		
//		ExtendedIterator conceptIt = ontology.listClasses();	//TODO better use listRDFTypes?
//		OntClass concept;
//		Individual individual;
		
//		while (conceptIt.hasNext()) {
//			concept = (OntClass) conceptIt.next();
//			Term termLit = new Term(concept.getLocalName(), concept.getLocalName(), concept.getURI(),Term.LITERAL);
//			Term termGlob = new Term(concept.getLocalName(), concept.getLocalName(), concept.getURI(),Term.STANDARD);
//			//TODO mischien beter rdf:about bevragen?
//			List<String> representationsLit = new ArrayList<String>();
//			List<String> representationsStandard = new ArrayList<String>();
//			getRepresentations(concept,representationsLit,representationsStandard);
//			termLit.setRepresentations(representationsLit);
//			nrOfStrings += addTermRepresentations(termLit,representationsLit);
//			nrOfStrings += addTermRepresentations(termGlob,representationsStandard);
//		}	
//		conceptIt = ontology.listIndividuals();
//		while (conceptIt.hasNext()) {
//			individual = (Individual) conceptIt.next();
//			concept = individual.getOntClass();
//			Term termLit = new Term(concept.getLocalName(), individual.getLocalName(), individual.getURI(),Term.LITERAL);
//			Term termGlob = new Term(concept.getLocalName(), individual.getLocalName(), individual.getURI(),Term.STANDARD);
//			//TODO mischien beter rdf:about bevragen?
//			List<String> representationsLit = new ArrayList<String>();
//			List<String> representationsStandard = new ArrayList<String>();
//			getRepresentations(individual,representationsLit,representationsStandard);
//			termLit.setRepresentations(representationsLit);
//			nrOfStrings += addTermRepresentations(termLit,representationsLit);
//			nrOfStrings += addTermRepresentations(termGlob,representationsStandard);
//		}
		
		ResIterator iter = ontology.listSubjects();
		
		while (iter.hasNext()) {
			Resource resource = (Resource) iter.next();
			Term termLit = new Term(resource.getLocalName(), resource.getLocalName(), resource.getURI(),Term.LITERAL);
			Term termGlob = new Term(resource.getLocalName(), resource.getLocalName(), resource.getURI(),Term.STANDARD);
//			//TODO mischien beter rdf:about bevragen?
			List<String> representationsLit = new ArrayList<String>();
			List<String> representationsStandard = new ArrayList<String>();
			getRepresentations(resource,representationsLit,representationsStandard);
			termLit.setRepresentations(representationsLit);
			nrOfStrings += addTermRepresentations(termLit,representationsLit);
			nrOfStrings += addTermRepresentations(termGlob,representationsStandard);
		}
		
		System.err.println("[APOLDA] " + new Integer(nrOfStrings).toString() + " " + STATUS_ENDREADONT);
	
		return nrOfStrings;
	}
	
	private void getRepresentations(Resource concept,
			List<String> representationsLit, List<String> representationsStand) {
		
		StmtIterator propIt;
		String strRepresentation;
		Locale lang = null;

		if (prefRepresentationAP == null && altRepresentationAP == null) {
			strRepresentation = concept.getLocalName(); //TODO check of getLocalName of toString doet wat het moet
			if (strRepresentation != null) {
				strRepresentation = strRepresentation.trim();
				addRepresentationToList(representationsLit,
						representationsStand, strRepresentation);
			}
		}

		if (prefRepresentationAP != null) {
			propIt = concept.listProperties(prefRepresentationAP);
			while (propIt.hasNext()) {
				Statement prop = propIt.nextStatement();
				
					if(language != null){
						lang = new Locale(prop.getLanguage());
					} else {
						lang = null;
					}
				if(lang == null || lang.getLanguage() == "" || lang.getLanguage() == language.getLanguage()){
					strRepresentation = prop.getString(); //TODO gevaarlijk! ten minste in een try-catch blok!
					addRepresentationToList(representationsLit,
							representationsStand, strRepresentation);
				}
			}


			if (altRepresentationAP != null) {
				propIt = concept.listProperties(altRepresentationAP);
				while (propIt.hasNext()) {
					Statement prop = propIt.nextStatement();
	
					if(language != null){
						lang = new Locale(prop.getLanguage());
					} else {
						lang = null;
					}
					if(lang == null || lang.getLanguage() == "" || lang.getLanguage() == language.getLanguage()){
						strRepresentation = prop.getString(); 
						addRepresentationToList(representationsLit,
								representationsStand, strRepresentation);
					}
				}
			} 
		} 
	}
	
	private void addTermRepresentation(String representation, Term term) {
		char currentChar;
		FSMState currentState = initialState;
		FSMState nextState;
		boolean isSpace;

		for (int i = 0; i < representation.length(); i++) {
			currentChar = representation.charAt(i);
			isSpace = Character.isWhitespace(currentChar);
			if (isSpace) {
				if(i > 0 && Character.isWhitespace(representation.charAt(i-1)))
					continue;
				else
					currentChar = ' ';
			} else
				currentChar = Character.toUpperCase(currentChar);
			nextState = currentState.next(currentChar);
			if (nextState == null) {
				nextState = new FSMState();
				fsmStates.add(nextState);
				currentState.put(currentChar, nextState);
				if (isSpace)
					nextState.put(' ', nextState);
			}
			currentState = nextState;
		} // for(int i = 0; i< representation.length(); i++)

		currentState.addTerm(term);
	}
	
	private void addRepresentationToList(List<String> representationsLit,
			List<String> representationsStand, String representation) {
		int len;

		len = representation.length();
		if (len > 2 && representation.charAt(0) == '"'
				&& representation.charAt(len - 1) == '"') {
			representation = representation.substring(1, len - 1);
			representationsLit.add(representation);
		} else {
			representationsStand.add(representation);
		}
	}
	
	private int addTermRepresentations(Term term, List<String> representations) {
		int rc = 0;
				
		if(representations != null) {
			Iterator<String> representation = representations.iterator();
			while (representation.hasNext()) {
				addTermRepresentation((String) representation.next(), term);
				rc++;
			}
		}
	
		return rc;
	}
	
	/**************************
	 * Deel 2
	 **************************/
	 
	private int findConcepts(Content theContent,JCas jCas) 
	throws AnnotatorProcessException {	 
		int nrOfAnnotations = 0;
		int iterations = 0;
		char currentChar;
		FSMState currentState = initialState;
		FSMState nextState;
		FSMState lastMatchingState = null;
		Region lastMatchingRegion = null;
		Set<Term> termSet;
		
		while (theContent.more()) {
			currentChar = theContent.getNextChar(currentState);						
			currentChar = Character.toUpperCase(currentChar);
			
			nextState = currentState.next(currentChar);
			if (nextState == null) { // the matching stopped	
				// Try backtracking in order to find a longer match
				currentState = theContent.redo();
				if(currentState == null) {
					// reset the FSM
					currentState = initialState;
					theContent.forward(currentState);
			
					// if we had a successful match then act on it;
					if (lastMatchingState != null && lastMatchingRegion != null) {
						termSet = checkMatchingTerms(jCas, lastMatchingRegion,lastMatchingState.getTermSet());
						if(termSet != null) {
							nrOfAnnotations += addAnnotation(jCas, termSet,	lastMatchingRegion);
							lastMatchingState = null;
						}
					}
				}					
			} else {// go on with the matching
				currentState = nextState;
				// if we have a successful state then store it
				if (currentState.isFinal() && theContent.atEndOfWord()) {
					Region r = theContent.getCurrentRegion();
					if(lastMatchingRegion == null || !lastMatchingRegion.contains(r)) {
						lastMatchingState = currentState;
						lastMatchingRegion = r;
					}
				}
			} 
	
			//TODO mogelijkheid tot afbreken implementeren
			//if((iterations++)%256 == 0 && isInterrupted()) {
				//throw new ExecutionInterruptedException(EXC_ABRUPT);
			//}
				
		} //while (theContent.more()) 
		if (lastMatchingState != null) {
			termSet = checkMatchingTerms(jCas,lastMatchingRegion,lastMatchingState.getTermSet());
			if(termSet != null)
				nrOfAnnotations += addAnnotation(jCas,termSet, lastMatchingRegion);
		}
		
		return nrOfAnnotations;
	}
	
	private Set<Term> checkMatchingTerms(JCas jCas, Region matchingRegion,Set<Term> termSet)
	throws AnnotatorProcessException {
		Set<Term> newTermSet = new HashSet<Term>(); 
		Iterator<Term> termSetIt = termSet.iterator();
		Term term;
		int begin;
		int end;
		String literalText = null;
		List<String> representations;
		
		while(termSetIt.hasNext()) {
			term = (Term) termSetIt.next();
			if(term.type == Term.STANDARD) {
				newTermSet.add(term);
			} else {
				if(literalText == null) {
					//try{
						begin = matchingRegion.getBegin();
						end = matchingRegion.getEnd() + 1;
						//literalText = document.getContent().getContent(begin,end).toString();
						literalText = jCas.getDocumentText().substring(begin, end);
						//TODO een beetje onhandig; hier kan een gigantische String geladen moeten worden
						//TODO matchingRegion is misschien ook slecht, een lijst van sofa's beter?
					//} catch(InvalidOffsetException e) {
						//throw new AnnotatorProcessException(e);
					//}
				}
				representations = term.getRepresentations();
				if(representations != null && representations.contains(literalText)) 
					newTermSet.add(term);
			}
		}
		return newTermSet;
	}
	
	public int analyze(JCas jCas, Feature lemmaFeature, Type tokenAnnotation) throws AnnotatorProcessException {
		int nrOfAnnotations = 0;

		try {
			//fireStatusChanged(STATUS_STARTDOC + " " + document.getName() + "...");
			//TODO something like the line above?
	
			Content theContent = new Content(jCas,lemmaFeature,tokenAnnotation);
			theContent.forward(initialState);
			
			nrOfAnnotations = findConcepts(theContent, jCas);
			
			return nrOfAnnotations;

		} catch (Exception e) {
			throw new AnnotatorProcessException(e);
		}
	}
	private int addAnnotation(JCas jCas, Set<Term> lookupSet, Region matchedRegion) {
		Term currentTerm;
		int nrOfAnnotations = 0;
		Iterator<Term> lookupIter = lookupSet.iterator();

		while (lookupIter.hasNext()) {
			currentTerm = (Term) lookupIter.next();
			
			
			OntologyAnnotation newAnnot = new OntologyAnnotation(jCas,
					matchedRegion.getBegin(), matchedRegion.getEnd());
			//newAnnot.setId(ID); //TODO waar haken we een ID vandaan?
			newAnnot.setOntologyURI(currentTerm.ontology); // TODO currentTerm.ontology andere naam geven?
			newAnnot.setConceptName(currentTerm.oClass);
			if (!currentTerm.oIdentifier.equals("")) {
				newAnnot.setInstanceName(currentTerm.oIdentifier);
			}

			newAnnot.addToIndexes();
		}// while(lookupIter.hasNext())

		return nrOfAnnotations;
	}
	
}
