package nl.telin.mymedia.apolda;

import org.apache.uima.TokenAnnotation;
import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.jcas.JCas;
import org.apache.uima.jcas.tcas.Annotation;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.Stack;

/**
 *
 * @author Telematica Instituut, Christian Wartena
 */

public class Content {
	private Annotation tokArray[];
	JCas jCas;
	private int charIdx = 0;
	private String tokenString = "";
	private Feature lemmaFeature;
	private Type tokenAnnotation;
	private Stack<Choicepoint>  choicepoints;
	FSIterator tokenIterator;
	FSIterator startTokenIterator;

	public Content(JCas jCas, Feature lemmaFeature, Type tokenAnnotation) {

		//TODO exceptions behandelen
		this.lemmaFeature = lemmaFeature;
		this.tokenAnnotation = tokenAnnotation;
		this.jCas = jCas;

		//tokenIterator = jCas.getCas().getAnnotationIndex(tokenAnnotation).iterator();

		choicepoints= new Stack<Choicepoint> ();
	}

	public boolean more() {
		return (charIdx < tokenString.length() || tokenIterator.hasNext());  //tokIdx+1 < nrOfTokens) 
	}

	public void forward(FSMState state){
		String lemmaString;

		if(startTokenIterator == null) {
			startTokenIterator = jCas.getCas().getAnnotationIndex(tokenAnnotation).iterator();
		} else {
			startTokenIterator.moveToNext(); //eerst checken met hasNext of is de volgende isValid() voldoende?
		}
		//tokenIterator.moveTo(startTokenIterator.get());
		tokenIterator = startTokenIterator.copy();

		choicepoints.clear();
		if(startTokenIterator.isValid()) { //TODO is dit voldoende?
			TokenAnnotation ta = (TokenAnnotation) startTokenIterator.get(); //TODO daarvoor hebben we eigenlijk this.tokenAnnotation
			tokenString = ta.getCoveredText(); //TODO is dit de juiste waarde?
			if(lemmaFeature != null){
				lemmaString = ta.getLemma(); //TODO eveneens weinig flexibel
				if(lemmaString != null && !lemmaString.equalsIgnoreCase(tokenString)) { //TODO there could be more than one possible stem. This should become a while-loop
					choicepoints.push(new Choicepoint(tokenIterator,state));
				} 
			}
		} else {
			tokenString = "";
		}
		charIdx = 0;
	}

	public FSMState redo() {
		if(choicepoints.isEmpty())
			return null;
		else {
			Choicepoint p = (Choicepoint) choicepoints.pop();
			if(p.getIterator() != null && lemmaFeature != null){
				tokenIterator = p.getIterator();
				TokenAnnotation ta = (TokenAnnotation) tokenIterator.get();
				tokenString = ta.getLemma();
			} 
			charIdx = 0;

			return p.getState();
		}
	}

	public boolean atEndOfWord() {
		return charIdx == tokenString.length();
	}

	public char getNextChar(FSMState state) {
		String lemmaString;
		char rc = ' ';
		if(charIdx < tokenString.length()) {
			rc = tokenString.charAt(charIdx);
			charIdx++;
		} else { //TODO hier zien we veel code herhaling. Een veeg teken
			tokenIterator.moveToNext();
			if(tokenIterator.isValid()) {
				TokenAnnotation ta = (TokenAnnotation) tokenIterator.get(); 
				tokenString = ta.getCoveredText();
				//TODO deze code porteren
				//if(!tokArray[tokIdx].getFeatures().get(ANNIEConstants.TOKEN_KIND_FEATURE_NAME).equals("word")) {
					//choicepoints.push(new Choicepoint(state));
				//} else {
					if(lemmaFeature != null) {
						lemmaString = ta.getLemma(); //TODO 
						//lemmaString = ta.getStringValue(lemmaFeature);
						if(lemmaString != null 
								&& !lemmaString.equals("")
								&& !lemmaString.equalsIgnoreCase(tokenString)) { //TODO there could be more than one possible stem. This should become a while-loop
							choicepoints.push(new Choicepoint(tokenIterator,state));
						} 
				//	}

					//TODO deze code porteren
					//if(!tokArray[tokIdx-1].getFeatures().get(ANNIEConstants.TOKEN_KIND_FEATURE_NAME).equals("word")) {
					//	choicepoints.push(new Choicepoint(state));
					//}
				}
			} else {
				tokenString = "";
			}
			charIdx = 0;
		}
		return rc;
	}

	public Region getCurrentRegion() {

		TokenAnnotation taB = (TokenAnnotation) startTokenIterator.get();
		TokenAnnotation taE = (TokenAnnotation) tokenIterator.get();
		return new Region(taB.getBegin(),taE.getEnd());
	}

}
