package nl.telin.mymedia.uima.annotator;

import java.util.Iterator;
import nl.telin.mymedia.analysis.infgain.SparseVector;

/**
 * @author Christian.Wartena
 * utility class for deriving statistics
 */
public class Statistics {
	
	/**
	 * calculate entropy given probabilities.
	 * @param probabilities (0.0 - 1.0)
	 * @return entropy
	 */
	public static float Entropy(float probabilities[]) {
		float E = 0;
		int s = probabilities.length;
		for (int i = 0; i < s; i++) {
			float p = probabilities[i];
			if (p > 0.0) {
				E += p * Math.log(1.0f / p);
			}
		}
		return E;
	}
	
	/**
	 * calculate JSDivergence given two sparsevectors.
	 * @param p1 vector one
	 * @param p2 vector two
	 * @return JSDivergence
	 */	
	public static float JSDivergence(SparseVector<Float> p1, SparseVector<Float> p2) {
		float E = 1.38629436111989061f;
		
		Iterator<Integer> it;
		if(p1.getNrOfNonZeroValues() < p2.getNrOfNonZeroValues()) {
			it = p1.getKeyIterator();
		} else {
			it = p2.getKeyIterator();
		}
		
		int t;
		while(it.hasNext()){
			t = it.next();
			float p1_t = p1.getFloatAt(t);
			float p2_t = p2.getFloatAt(t);
			if(p1_t > 0 && p2_t > 0) {
				float sum = p1_t + p2_t;
				E += p1_t * Math.log(p1_t/sum);
				E += p2_t * Math.log(p2_t/sum);
					
				/*
			  	E += sum * Math.log(1f/sum);
				E -= p1_t * Math.log(1f/p1_t);
				E -= p2_t * Math.log(1f/p2_t);
				*/
			} 
		}
		
		return E;
	}
}

