package nl.telin.mymedia.uima.annotator;

import java.util.LinkedList;
import java.util.Queue;

import org.apache.uima.TokenAnnotation;
import org.apache.uima.analysis_engine.ResultSpecification;
import org.apache.uima.analysis_engine.annotator.AnnotatorConfigurationException;
import org.apache.uima.analysis_engine.annotator.AnnotatorInitializationException;
import org.apache.uima.analysis_engine.annotator.AnnotatorProcessException;
import org.apache.uima.analysis_engine.annotator.JTextAnnotator_ImplBase;
import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.TypeSystem;
import org.apache.uima.jcas.JCas;


/**
 * @author Christian.Wartena
 */
public class VerbalConceptAnnotator extends JTextAnnotator_ImplBase {

	private static final String TOKEN_ANNOTATION_NAME = "org.apache.uima.TokenAnnotation";
	private Type tokenAnnotation;
	private Queue<TokenAnnotation> tokenQueue;
	
	@Override
	public void typeSystemInit(TypeSystem aTypeSystem)
			throws AnnotatorInitializationException,
			AnnotatorConfigurationException {
	
		// initialize cas token type
		this.tokenAnnotation = aTypeSystem.getType(TOKEN_ANNOTATION_NAME);
		this.tokenQueue = new LinkedList<TokenAnnotation>();			
		
	}
	
	//@Override
	/**
	 * @inheritDoc
	 */
	public void process(JCas cas, ResultSpecification resultSpec)
			throws AnnotatorProcessException {
		// TODO Auto-generated method stub
		FSIterator tokenIterator = cas.getCas().getAnnotationIndex(
				this.tokenAnnotation).iterator();
		
		while (tokenIterator.hasNext()) {
			// get token content
			TokenAnnotation annot = (TokenAnnotation) tokenIterator.next();
			if(annot.getPos() != null && (
				annot.getPos().equals("VBD")
				|| annot.getPos().equals("VBZ")
				|| annot.getPos().equals("VB"))) {
				String lemma = annot.getLemma();
				//lemma = lemma + "ing";
				lemma = makeGerund(lemma);
				annot.setLemma(lemma);
			}
		}
	}

	private String makeGerund(String lemma) {
		// TODO Auto-generated method stub
		String gerund = new String();
		
		
		if(lemma.endsWith("e")) {
			char secLast = lemma.charAt(lemma.length()-2);
			if("klmnrst".indexOf(secLast) >= 0) {
				lemma = lemma.substring(0, lemma.length()-1);
			}
			
		}
		
			
		gerund = lemma + "ing";
		
		return gerund;
	}

}
