package nl.telin.mymedia.ontology;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Selector;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

/**
 * @author Wout.Slakhorst
 * Counts relations between ontologt concepts
 */
public class RelationCounter {
	private InfModel ontologyInf;
	private Map<Integer, Integer> count;
	private Set<String> terms;
	private Map<String, Set<String>> distance1;
	
	private Model model;

	public RelationCounter() {
		count = new HashMap<Integer, Integer>();
		terms = new HashSet<String>();
	}

	public static void main(String[] args) throws Exception {
		RelationCounter wc = new RelationCounter();
		if (args.length != 1) {
			printUsage();
			System.exit(1);
		}
		wc.loadModel(args[0]);
		wc.count();		
	}

	public static void printUsage() {
		System.out.println("Usage: ");
		System.out.println("java WeightCalculator PATH_TO_ONTOLOGY");
	}

	public void writeModel(String outputFile) throws IOException {
		model.write(new FileWriter(outputFile));
	}

	public void loadModel(String ontology) throws FileNotFoundException {
		try {
			InputStream stream = new FileInputStream(ontology);

			// load jena
			model = ModelFactory.createDefaultModel();
			model.read(stream, "");
			ontologyInf = ModelFactory.createRDFSModel(model);

			ResIterator iter = model.listSubjects();

			while (iter.hasNext()) {
				Resource resource = (Resource) iter.next();
				terms.add(resource.getURI());
			}

		} catch (FileNotFoundException e) {
			throw e;
		}
	}

	public void count() {
		findAllDistance1Subjects();

		// total = 0;
		//double count = 0;
		
		for (String subject : terms) {
			int c = distance1Relations(subject);
			
			Integer c2 = count.get(c);
			if (c2 == null) {
				c2 = new Integer(0);
			}
			c2++;
			count.put(c, c2);
			//count++;
			//total += weight;
		}
		
		for (Map.Entry<Integer, Integer> entry: count.entrySet()) {
			System.out.println(entry.getKey() + ", " + entry.getValue());
		}

	}

	private int distance1Relations(String subject) {
		int rc = 0;
		Set<String> relatedTerms = distance1.get(subject);

		for (String term : relatedTerms) {
			if (terms.contains(term)) {
				rc++;
			}
		}

		return rc;
	}

	private void findAllDistance1Subjects() {
		distance1 = new HashMap<String, Set<String>>();
		for (String subject : terms) {
			distance1.put(subject, getDistance1Subjects(subject));
		}
	}

	/**
	 * returns a set of subjects, each with the namespace
	 * 
	 * @param subject
	 *            with namespace
	 * @return
	 */
	private Set<String> getDistance1Subjects(String subject) {

		Set<String> result = new HashSet<String>();

		// related
		Selector query = new SimpleSelector(
				ontologyInf.createResource(subject), ontologyInf
						.createProperty("http://www.w3.org/2004/02/skos/core#",
								"related"), (RDFNode) null);
		StmtIterator iter = ontologyInf.listStatements(query);
		while (iter.hasNext()) {
			Statement stmt = iter.nextStatement();
			result.add(stmt.getObject().toString());
		}

		// narrower
		query = new SimpleSelector(ontologyInf.createResource(subject),
				ontologyInf.createProperty(
						"http://www.w3.org/2004/02/skos/core#", "narrower"),
				(RDFNode) null);
		iter = ontologyInf.listStatements(query);
		while (iter.hasNext()) {
			Statement stmt = iter.nextStatement();
			result.add(stmt.getObject().toString());
		}

		// broader
		query = new SimpleSelector(ontologyInf.createResource(subject),
				ontologyInf.createProperty(
						"http://www.w3.org/2004/02/skos/core#", "broader"),
				(RDFNode) null);
		iter = ontologyInf.listStatements(query);
		while (iter.hasNext()) {
			Statement stmt = iter.nextStatement();
			result.add(stmt.getObject().toString());
		}

		result.remove(subject);

		return result;
	}	
}
