package nl.telin.mymedia.tvanytime;

import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

/**
 * @author Wout.Slakhorst
 * 
 * Extracts Synopsis from TvAnytime files
 */
public class TvAnytimeSynopsisExtractor {
	
	private String inputFile;
	private String outputDir;
	
	public TvAnytimeSynopsisExtractor(String inputFile, String outputDir) {
		this.inputFile = inputFile;
		this.outputDir = outputDir;
	}
	
	public void run() {
		try {
			FileInputStream fileInputStream = 
		        new FileInputStream(inputFile);
		    XMLStreamReader xmlStreamReader =
		        XMLInputFactory.newInstance().
		            createXMLStreamReader(fileInputStream);
		    
		    //String docName = null;
		    //String elementName = null;
		    int count = 0;
		    FileWriter writer = null;
		    
		    while (xmlStreamReader.hasNext()) {
		    	int code = xmlStreamReader.next();
		    	switch (code) {
		    	case XMLStreamConstants.START_ELEMENT:
		    		if(xmlStreamReader.getLocalName().equals("Title")){
		    			//xmlStreamReader.next();
		    			String docName = xmlStreamReader.getElementText().replace("?", "").replace("\\", "").replace("/","").replace(" ", "_");
		    			String filename = outputDir + "/Synopsis_" + docName + "_" + count++ + ".txt";
		    			writer = new FileWriter(filename);
		    		} else if (xmlStreamReader.getLocalName().equals("Synopsis")) {
		    			//xmlStreamReader.next();
		    			String text = xmlStreamReader.getElementText();
		    			writer.write(text);
		    			writer.write("\n");
		    			writer.write("\n");
		    		}
		    		break;
		    	case XMLStreamConstants.END_ELEMENT:
		    		if (xmlStreamReader.getLocalName().equals("ProgramInformation")) {
		    			writer.close();
		    		}
		    	}
		    }
		    
		} catch (IOException e) {
			e.printStackTrace();
		} catch (XMLStreamException e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String args[]) throws Exception {
		if (args.length < 2) {
			printUsage();
			System.exit(1);
		}

		TvAnytimeSynopsisExtractor ext = new TvAnytimeSynopsisExtractor(args[0], args[1]);
		ext.run();
	}

	private static void printUsage() {
		System.out.println("Usage: java TvAnytimeSynopsisExtractor [input.xml] [output_dir]");

	}
}
