﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Windows.Forms;
using System.Linq;
using System.Text;

using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Framework;
using EMIC.Design;

using MovieLens;

namespace RecommenderDisplay
{
    public class FeedbackDisplay
    {
        public const double MaxRating = 5.0;
        public const double MinRating = 1.0;

        private MovieLensRecommender recommender;
        public CatalogItem catalogItem;
        private RatingAction feedbackItem;
        private Prediction<CatalogItem> prediction;
        private User userId;

        internal FeedbackDisplay(
            MovieLensRecommender recommender, 
            CatalogItem item, 
            RatingAction feedback,
            Prediction<CatalogItem> prediction,
            User user)
        {
            Contract.Precondition(null != recommender);
            Contract.Precondition(null != item);
            Contract.Precondition(null != user);
            //Feedback can be null.

            this.recommender = recommender;
            this.catalogItem = item;
            this.feedbackItem = feedback;
            this.prediction = prediction;
            this.userId = user;
        }

        public string Title
        {
            get
            {
                // if current culture's language is not found, an arbitrary language will be used
                if (this.catalogItem.ItemTitles != null && this.catalogItem.ItemTitles.Count > 0){
                    if (this.catalogItem.ItemTitles.Count(item => item.Language.Code == CultureInfo.CurrentCulture.Name) > 0)
                    {
                        return this.catalogItem.ItemTitles.First(item => item.Language.Code == CultureInfo.CurrentCulture.Name).Title;
                    }
                    else
                    {
                        // return data for an arbitrary language
                        return this.catalogItem.ItemTitles.First().Title;
                    }
                }
                return null;
            }
        }

        public double PredictedRating
        {
            get
            {
                return RatingAction.ScaleRatingToApplication(this.prediction.Rating, FeedbackDisplay.MinRating, FeedbackDisplay.MaxRating);
            }
        }

        public string ActualRating
        {
            get
            {
                if (this.feedbackItem != null)
                {
                    return this.feedbackItem.ScaleRatingToApplication(1.0, 5.0).ToString();
                }
                else
                {
                    return string.Empty;
                }
            }
            set
            {
                double newValue = double.Parse(value);
                if (newValue <= FeedbackDisplay.MaxRating && newValue >= FeedbackDisplay.MinRating)
                {
                    this.feedbackItem = new RatingAction(this.userId, this.catalogItem, RatingAction.ScaleRatingToRecommender(newValue, FeedbackDisplay.MinRating, FeedbackDisplay.MaxRating));
                    try
                    {
                        recommender.ProcessFeedback(new UserAction[] {this.feedbackItem});
                    }
                    catch (Exception e)
                    {
                        TraceLog.OnException(e);
                        this.ActualRating = null;
                        MessageBox.Show("Invalid User.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
                else 
                if (null == value)
                {
                    this.feedbackItem = null;
                }
                else
                {
                    string message = string.Format("Ratings must be in the range {0}, {1}, inclusive.", FeedbackDisplay.MinRating, FeedbackDisplay.MaxRating);
                    MessageBox.Show(message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }
    }
}
