﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Framework;
using MyMediaProject.RecommenderSystem.Framework.Metrics;
using MovieLens;

namespace MovieLensRecommenderMetrics
{
    class Program
    {
        static Guid metricProgramGuid = new Guid("{EA1B8A73-9F63-4710-8847-CFC359795556}");
        static ProgramId metricProgramId = new ProgramId(metricProgramGuid, "Metric tests for the MovieLens database");

        static void Main(string[] args)
        {
            using (MovieLensRecommender recommender = new MovieLensRecommender())
            {
                recommender.Initialize();
                using (RecommenderMetrics<RatingAction, RatingAction> metric = new RecommenderMetrics<RatingAction, RatingAction>(
                    metricProgramId,
                    recommender,
                    new RandomAllButNTestSet<RatingAction>(recommender, 1),
                    new MetricBase<RatingAction>[] { 
                        new RootMeanSquaredErrorMetric<RatingAction>(),
                        new ExpectedUtilityMetric<RatingAction>(0.0, 5)}))
                {
                    metric.PartitionTestSet();
                    recommender.Train();
                    ReadOnlyDictionary<RecommenderId, double> results = metric.ProduceMetrics(recommender.DefaultFlowId);
                    foreach (KeyValuePair<RecommenderId, double> result in results)
                    {
                        Console.WriteLine("{0} produced result {1}", result.Key, result.Value);
                    }
                    metric.RemoveAllTestFeedback();
                    Console.WriteLine("Press any key to finish.");
                    Console.ReadKey();
                }
            }
        }
    }
}
