﻿using System;
using System.Collections.Specialized;
using ExternalSystemIntegration;
using System.Collections.Generic;
using System.Data.Linq;
using System.Windows.Forms;
using EMIC.Design;
using ExtremeSwank.OpenId;
using System.Collections.ObjectModel;
using ExtremeSwank.OpenId.PlugIns.Extensions;
using MyMediaProject.RecommenderSystem.Core;
using OpenIdIntegrationComponent;

namespace OpenIdTestGUI
{
    /// <summary>
    /// Utility to test and demonstrate the functionality of the OpenID Integration component
    /// </summary>
    public partial class OpenIdTestGuiForm : Form
    {
        private bool IsExtensionUsageEnabled = false;
        private OpenIdIntegrator Integrator = new OpenIdIntegrator();
        private WebBrowserRedirectRequest lastRedirectRequest;
        private List<String> optionalSimpleExtensionFields = new List<String>();
        private List<String> requiredSimpleExtensionFields = new List<String>();
        private String LastClaimedIdentity = "";

        /// <summary>
        /// Constructor, initializing the form
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1805:DoNotInitializeUnnecessarily")]
        public OpenIdTestGuiForm()
        {
            InitializeComponent();
            UseExtensionCheckBox.Enabled = true;
        }


        /// <summary>
        /// Generate authentication URL
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAuthenticate_Click(object sender, EventArgs e)
        {
            if (UseExtensionCheckBox.Checked)
            {
                IsExtensionUsageEnabled = true;
            }

            this.LastClaimedIdentity = textBoxOpenId.Text;

            // the value must be consistent with the one that will be used for processing callback
            UseExtensionCheckBox.Enabled = false;

            Dictionary<String, Object> authenticationParameters = new Dictionary<String, Object>();
            authenticationParameters.Add(PerformAuthenticationParameterKeyNames.Identity, textBoxOpenId.Text);

            if (IsExtensionUsageEnabled)
            {    
                requiredSimpleExtensionFields.Add(SimpleRegistrationFields.FullName);
                requiredSimpleExtensionFields.Add(SimpleRegistrationFields.Gender);
                requiredSimpleExtensionFields.Add(SimpleRegistrationFields.Country);
                requiredSimpleExtensionFields.Add(SimpleRegistrationFields.Language);
                
                optionalSimpleExtensionFields.Add(SimpleRegistrationFields.Email);
                optionalSimpleExtensionFields.Add(SimpleRegistrationFields.Nickname);
                optionalSimpleExtensionFields.Add(SimpleRegistrationFields.PostalCode);
                optionalSimpleExtensionFields.Add(SimpleRegistrationFields.TimeZone);
                optionalSimpleExtensionFields.Add(SimpleRegistrationFields.DateOfBirth);

                authenticationParameters.Add(PerformAuthenticationParameterKeyNames.RequiredSimpleRegistrationFieldsList, requiredSimpleExtensionFields);
                authenticationParameters.Add(PerformAuthenticationParameterKeyNames.OptionalSimpleRegistrationFieldsList, optionalSimpleExtensionFields);
            }

            ExternalSystemIntegratorResult resultObj = Integrator.PerformExternalAuthentication(authenticationParameters);
            this.lastRedirectRequest = resultObj.GetRedirectRequestResult();
            textBoxRedirectTo.Text = "";
            if (lastRedirectRequest.Uri != null)
            {
                textBoxRedirectTo.Text = lastRedirectRequest.Uri.ToString();
            }
        }

        /// <summary>
        /// Process OpenID provider's callback
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void buttonProcessResponse_Click(object sender, EventArgs e)
        {
            OpenIdUser theUser = null;
            StatusLogTextBox.Text = "";

            try{
                TraceLog.Write(LogEventType.Verbose, "buttonProcessResponse_Click with CallBack URL = '{0}'.", new object[] { FallbackUrlTextBox.Text });
                ExternalSystemIntegratorResult Result = Integrator.ProcessProvidersAuthenticationCallback(this.LastClaimedIdentity, FallbackUrlTextBox.Text, new ReadOnlyCollection<String>(requiredSimpleExtensionFields), new ReadOnlyCollection<String>(optionalSimpleExtensionFields));
                
                if (!Result.IsOtherObjectResultType())
                {
                    MessageBox.Show("Authentication Failed.");
                    return;
                }

                theUser = (OpenIdUser)Result.GetObjectResult();
            }
            catch (InvalidCastException)
            {
                MessageBox.Show("Authentication Failed (user is null).");
                return;
            }
            catch (ExternalSystemIntegratorException ex)
            {
                MessageBox.Show("Authentication Failed: \r\n" + ex.ToString());
                return;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Authentication Failed: \r\n" + ex.ToString());
                return;
            }

            StatusLogTextBox.Text += "Identity = " + theUser.Identity + "\r\n";

            if (IsExtensionUsageEnabled)
            {
                StatusLogTextBox.Text += "\r\nExtensionData:\r\n";

                foreach (String key in theUser.ExtensionData.Keys)
                {
                    if (theUser.ExtensionData[key] == null)
                    {
                        continue;
                    }
                    StatusLogTextBox.Text += key + " = '" + theUser.ExtensionData[key] + "'.\r\n";
                }
            }
        }

        
        /// <summary>
        /// Exit the program
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void textBoxOpenId_TextChanged(object sender, EventArgs e)
        {
            UseExtensionCheckBox.Enabled = true;
        }
    }
}
