﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Core.Config;
using MyMediaProject.RecommenderSystem.Core;

namespace ExternalSystemIntegration
{
    /// <summary>
    /// Utility class that can be used to create ExternalSystemIntegration tables
    /// </summary>
    public static class CreateDatabaseUtility
    {
        /// <summary>
        /// Creates MyMedia tables, if do not exist
        /// </summary>
        public static void CreateExternalSystemIntegrationDatabaseIfMissing()
        {
            String connectionString = ConfigManager.DBConnectionString;
            CreateExternalSystemIntegrationDatabaseIfMissing(connectionString);
        }

        /// <summary>
        /// Creates ExternalSystemIntegrator tables, if missing
        /// </summary>
        /// <param name="connectionString">Connection string to initialize MyMedia DB context</param>
        public static void CreateExternalSystemIntegrationDatabaseIfMissing(String connectionString)
        {
            MyMediaDataContext context = new MyMediaDataContext(connectionString);
            CreateExternalSystemIntegrationDatabaseIfMissing(context);
        }


        /// <summary>
        /// Creates ExternalSystemIntegrator tables, if mising
        /// </summary>
        /// <param name="context">Initialized MyMedia DB context object</param>
        public static void CreateExternalSystemIntegrationDatabaseIfMissing(DataContext context)
        {
            if (context.DatabaseExists())
            {
                return;
            }
            
            context.CreateDatabase();
        }

        /// <summary>
        /// Creates ExternalSystemIntegrator tables, if missing
        /// </summary>
        /// <param name="connectionString">Connection string to initialize MyMedia DB context</param>
        public static void RecreateExternalSystemIntegrationDatabaseIfMissing(String connectionString)
        {
            MyMediaDataContext context = new MyMediaDataContext(connectionString);
            if (context.DatabaseExists())
            {
                context.DeleteDatabase();
                // Linq's bug: reinitialize the context.
                context.Dispose();
                context = new MyMediaDataContext(connectionString);

            }

            context.CreateDatabase();
        }

    }
}
