﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EMIC.Design;

namespace ExternalSystemIntegration
{
    /// <summary>
    /// Exception class that is used by external systems' integration components to handle exceptions
    /// </summary>
    [Serializable]
    public class ExternalSystemIntegratorException : SystemException 
    {
        /// <summary>
        /// Exceptions that are specific to external systems' integration components should be created via this constructor
        /// </summary>
        /// <param name="errorMessage">textual error message to be logged</param>
        public ExternalSystemIntegratorException(String errorMessage)
            : base(errorMessage)
        {
            TraceLog.Write(LogEventType.Error, "ExternalSystemIntegratorException: {0}.", new object[]{errorMessage});
        }


        /// <summary>
        /// Typical exceptions like System.ArgumentException should be processed via this constructor
        /// </summary>
        /// <param name="ex">Exception object with exception's data</param>
        public ExternalSystemIntegratorException(Exception ex): base(ex.ToString(), ex)
        {
            TraceLog.Write(LogEventType.Error, "ExternalSystemIntegratorException: " + ex.ToString());
        }
    }
}
