﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ExternalSystemIntegration
{
    /// <summary>
    /// External systems that can be used for authenticating users externally (and possibly provide more operations),
    /// should implement this interface.
    /// </summary>
    public interface IExternalSystemIntegrator
    {
        /// <summary>
        /// Performs necessary steps to authenticate user at an external social network. 
        /// Different derived classes might use distinct approaches.
        /// </summary>
        /// <param name="dictionaryParameters">
        /// Name value pairs of the parameters needed by implementations(e.g. fallback URL, username and password pair).
        /// The names of the parameters are defined by implementations.
        /// </param>
        /// <returns>ExternalSystemIntegratorResult which could be WebBrowserRedirectRequest</returns>
        ExternalSystemIntegratorResult PerformExternalAuthentication(Dictionary<String, Object> dictionaryParameters);
    }
}
